@@ -45,9 +45,6 @@ int AddressX86Flat32::byteSize()
 
 int AddressX86Flat32::compareTo(const Object *obj) const
 {
-/*	if (getObjectID() != to->getObjectID()) {
-		int as=1;
-	}*/
 	assert(getObjectID() == obj->getObjectID());
 	if (addr > ((AddressX86Flat32 *)obj)->addr) return 1;
 	if (addr < ((AddressX86Flat32 *)obj)->addr) return -1;
@@ -463,7 +460,7 @@ ObjectID AnalyX86Disassembler::getObjectID() const
 	return ATOM_ANALY_X86;
 }
 
-Address *AnalyX86Disassembler::createAddress(uint16 segment, uint32 offset)
+Address *AnalyX86Disassembler::createAddress(uint16 segment, uint64 offset)
 {
 	if (flags & (ANALYX86DISASSEMBLER_FLAGS_FLAT64 | ANALYX86DISASSEMBLER_FLAGS_AMD64)) {
 		return new AddressFlat64(offset);
@@ -521,11 +518,7 @@ Address *AnalyX86Disassembler::branchAddr(OPCODE *opcode, branch_enum_t branchty
 	x86dis_insn *o = (x86dis_insn*)opcode;
 	assert(o->op[1].type == X86_OPTYPE_EMPTY);
 	switch (o->op[0].type) {
-		case X86_OPTYPE_IMM: {
-/*          	if (o->op[0].imm == 0x1012c0f) {
-				int as=0;
-			}*/
-			
+		case X86_OPTYPE_IMM: {			
 			uint16 seg = 0;
 			if (flags & ANALYX86DISASSEMBLER_FLAGS_SEGMENTED) {
 				seg = getSegment(analy->addr);
@@ -573,6 +566,7 @@ Address *AnalyX86Disassembler::branchAddr(OPCODE *opcode, branch_enum_t branchty
 				return addr;
 			}
 		}
+		default: break;
 	}
 	return new InvalidAddress();
 }
@@ -583,7 +577,7 @@ Address *AnalyX86Disassembler::branchAddr(OPCODE *opcode, branch_enum_t branchty
 void	AnalyX86Disassembler::examineOpcode(OPCODE *opcode)
 {
 	x86dis_insn *o = (x86dis_insn*)opcode;
-	for (int i=0; i<3; i++) {
+	for (int i = 0; i < 5; i++) {
 		x86_insn_op *op = &o->op[i];
 		Address *addr = NULL;
 		taccess access;
@@ -614,6 +608,7 @@ void	AnalyX86Disassembler::examineOpcode(OPCODE *opcode)
 				}
 			}
 			break;
+		default: continue;
 		}
 		if (addr) {
 			if (analy->validAddress(addr, scvalid)) {
