@@ -3,7 +3,7 @@
  *	x86asm.cc
  *
  *	Copyright (C) 1999-2002 Stefan Weyergraf
- *	Copyright (C) 2005-2006 Sebastian Biallas (sb@biallas.net)
+ *	Copyright (C) 2005-2007 Sebastian Biallas (sb@biallas.net)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -19,7 +19,6 @@
  *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-
 #include <stdlib.h>
 #include <string.h>
 
@@ -38,6 +37,10 @@ enum {
 	X86ASM_PREFIX_F20F38,
 	X86ASM_PREFIX_0F3A,
 	X86ASM_PREFIX_660F3A,
+	X86ASM_PREFIX_0F7A,
+	X86ASM_PREFIX_0F7B,
+	X86ASM_PREFIX_0F24,
+	X86ASM_PREFIX_0F25,
 	X86ASM_PREFIX_D8,
 	X86ASM_PREFIX_D9,
 	X86ASM_PREFIX_DA,
@@ -133,29 +136,29 @@ static const x86addrcoding modrm32[3][8] = {
 };
 
 /* convert logical operand types to hardware operand types */
-static const int lop2hop[12][8] = {
+static const byte lop2hop[12][9] = {
 	/* X86_OPTYPE_EMPTY */
 	{},
 	/* X86_OPTYPE_IMM */
-	{TYPE_I, TYPE_Is, TYPE_J, TYPE_A, TYPE_Ix},
+	{TYPE_I, TYPE_Is, TYPE_J, TYPE_A, TYPE_Ix, TYPE_I4},
 	/* X86_OPTYPE_REG */
 	{TYPE_R, TYPE_Rx, TYPE_RXx, TYPE_G, TYPE_E, TYPE_MR},
 	/* X86_OPTYPE_SEG */
 	{TYPE_S, TYPE_Sx},
 	/* X86_OPTYPE_MEM */
-	{TYPE_E, TYPE_M, TYPE_O, TYPE_Q, TYPE_W, TYPE_MR},
+	{TYPE_E, TYPE_M, TYPE_MR, TYPE_O, TYPE_Q, TYPE_W, TYPE_VS, TYPE_X},
 	/* X86_OPTYPE_CRX */
 	{TYPE_C},
 	/* X86_OPTYPE_DRX */
 	{TYPE_D},
-	/* X86_OPTYPE_TRX */
-	{TYPE_T},
 	/* X86_OPTYPE_STX */
 	{TYPE_F, TYPE_Fx},
 	/* X86_OPTYPE_MMX */
 	{TYPE_P, TYPE_Q, TYPE_PR},
 	/* X86_OPTYPE_XMM */
-	{TYPE_V, TYPE_W, TYPE_VR, TYPE_Vx},
+	{TYPE_V, TYPE_W, TYPE_VR, TYPE_Vx, TYPE_VV, TYPE_VI, TYPE_VS, TYPE_VD},
+	/* X86_OPTYPE_YMM */
+	{TYPE_Y, TYPE_X, TYPE_YR, TYPE_YV, TYPE_YI},
 	/* X86_OPTYPE_FARPTR */
 	{},
 };
@@ -177,10 +180,11 @@ static const char immhsz64_64[] = { SIZE_V, 0 };
 
 static const char hsz8_16[] = { SIZE_B, 0 };
 static const char hsz16_16[] = { SIZE_W, SIZE_V, SIZE_VV, 0 };
-static const char hsz32_16[] = { SIZE_D, SIZE_P, SIZE_Z, 0 };
+static const char hsz32_16[] = { SIZE_D, SIZE_P, SIZE_Z, SIZE_R, 0 };
 static const char hsz48_16[] = { 0 };
 static const char hsz64_16[] = { SIZE_Q, SIZE_U, SIZE_Z, 0};
 static const char hsz128_16[] = { SIZE_O, SIZE_U, 0};
+static const char hsz256_16[] = { SIZE_Y, 0};
 
 static const char hsz8_32[] = { SIZE_B, 0 };
 static const char hsz16_32[] = { SIZE_W, 0 };
@@ -188,6 +192,7 @@ static const char hsz32_32[] = { SIZE_D, SIZE_V, SIZE_VV, SIZE_R, SIZE_Z, 0 };
 static const char hsz48_32[] = { SIZE_P, 0 };
 static const char hsz64_32[] = { SIZE_Q, SIZE_U, SIZE_Z, 0};
 static const char hsz128_32[] = { SIZE_O, SIZE_U, 0};
+static const char hsz256_32[] = { SIZE_Y, 0};
 
 static const char hsz8_64[] = { SIZE_B, 0 };
 static const char hsz16_64[] = { SIZE_W, 0 };
@@ -195,6 +200,7 @@ static const char hsz32_64[] = { SIZE_D, SIZE_Z, 0 };
 static const char hsz48_64[] = { 0 };
 static const char hsz64_64[] = { SIZE_Q, SIZE_U, SIZE_V, SIZE_VV, SIZE_R, SIZE_Z, 0};
 static const char hsz128_64[] = { SIZE_O, SIZE_U, 0};
+static const char hsz256_64[] = { SIZE_Y, 0};
 
 static const int reg2size[4] = {1, 2, 4, 8};
 static const int addr2size[4] = {-1, 2, 4, 8};
@@ -208,7 +214,23 @@ x86asm::x86asm(X86OpSize o, X86AddrSize a)
 {
 	opsize = o;
 	addrsize = a;
-	x86_insns = &x86_32_insns;
+	if (a != X86_ADDRSIZE64) {
+		prepInsns();
+	}
+}
+
+x86opc_insn (*x86asm::x86_32a_insns)[256];
+
+void x86asm::prepInsns()
+{
+	if (!x86_32a_insns) {
+		x86_32a_insns = ht_malloc(sizeof *x86_32a_insns);
+		memcpy(x86_32a_insns, x86_32_insns, sizeof x86_32_insns);
+	
+		(*x86_32a_insns)[0xc4] = x86_les;
+		(*x86_32a_insns)[0xc5] = x86_lds;
+	}
+	x86_insns = x86_32a_insns;
 }
 
 asm_insn *x86asm::alloc_insn()
@@ -358,14 +380,6 @@ asm_code *x86asm::encode(asm_insn *asm_insn, int options, CPU_ADDR cur_address)
 {
 	Assembler::encode(asm_insn, options, cur_address);
 	x86asm_insn *insn = (x86asm_insn*)asm_insn;
-	
-	addrsize_depend = false;
-	for (int i=0; i < 3; i++) {
-		if (insn->op[i].type == X86_OPTYPE_MEM) {
-			addrsize_depend = true;
-			break;
-		}
-	}
 		
 	newcode();
 	namefound = false;
@@ -377,6 +391,8 @@ asm_code *x86asm::encode(asm_insn *asm_insn, int options, CPU_ADDR cur_address)
 	esizes[0] = 0;
 	esizes[1] = 0;
 	esizes[2] = 0;
+	esizes[3] = 0;
+	esizes[4] = 0;
 	ambiguous = false;
 	match_opcodes(*x86_insns, insn, X86ASM_PREFIX_NO, MATCHOPNAME_MATCH);
 	if (!namefound && insn->repprefix != X86_PREFIX_NO) {
@@ -392,6 +408,11 @@ asm_code *x86asm::encode(asm_insn *asm_insn, int options, CPU_ADDR cur_address)
         	match_opcodes(x86_opc_group_insns[2], insn, X86ASM_PREFIX_F20F38, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
         	match_opcodes(x86_opc_group_insns[3], insn, X86ASM_PREFIX_0F3A, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
         	match_opcodes(x86_opc_group_insns[4], insn, X86ASM_PREFIX_660F3A, MATCHOPNAME_MATCH_IF_OPPREFIX);
+        	match_opcodes(x86_opc_group_insns[5], insn, X86ASM_PREFIX_0F7A, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+        	match_opcodes(x86_opc_group_insns[6], insn, X86ASM_PREFIX_0F7B, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+        	match_opcodes(x86_opc_group_insns[7], insn, X86ASM_PREFIX_0F24, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+        	match_opcodes(x86_opc_group_insns[8], insn, X86ASM_PREFIX_0F25, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+        	match_vex_opcodes(insn);
 	}
 	if (error) {
 		free_asm_codes();
@@ -413,7 +434,7 @@ bool x86asm::encode_insn(x86asm_insn *insn, x86opc_insn *opcode, int opcodeb, in
 	disppos = 0;
 
 	bool opsize_depend = false;
-	for (int i = 0; i < 3; i++) {
+	for (int i = 0; i < 4; i++) {
 		switch (x86_op_type[opcode->op[i]].size) {
 		case SIZE_BV:
 		case SIZE_V:
@@ -427,7 +448,7 @@ bool x86asm::encode_insn(x86asm_insn *insn, x86opc_insn *opcode, int opcodeb, in
 	code.context = (void*)opsize_depend;
 	
 	/* test rex thingies */
-	for (int i=0; i<3; i++) {
+	for (int i=0; i < 4; i++) {
 		if (insn->op[i].need_rex) {
 			rexprefix |= 0x40;
 		}
@@ -438,6 +459,8 @@ bool x86asm::encode_insn(x86asm_insn *insn, x86opc_insn *opcode, int opcodeb, in
 
 	modrmv = -1;
 	sibv = -1;
+	drexdest = -1;
+	drexoc0 = -1;
 	dispsize = 0;
 	immsize = 0;
 	if (additional_opcode != -1) {
@@ -512,7 +535,11 @@ bool x86asm::encode_insn(x86asm_insn *insn, x86opc_insn *opcode, int opcodeb, in
 	}
 
 	int rexpos = code.size;
-	if (rexprefix & 0x40) emitbyte(0xff); // dummy value
+	if ((rexprefix & 0x40)
+	 && prefix != X86ASM_PREFIX_0F24
+	 && prefix != X86ASM_PREFIX_0F25) {
+		emitbyte(0xff); // dummy value
+	}
 	
 	/* write opcodeprefixes and opcode */
 	switch (prefix) {
@@ -535,6 +562,22 @@ bool x86asm::encode_insn(x86asm_insn *insn, x86opc_insn *opcode, int opcodeb, in
 		emitbyte(0x0f);
 		emitbyte(0x3a);
 		break;
+	case X86ASM_PREFIX_0F24:
+		emitbyte(0x0f);
+		emitbyte(0x24);
+		break;
+	case X86ASM_PREFIX_0F25:
+		emitbyte(0x0f);
+		emitbyte(0x25);
+		break;
+	case X86ASM_PREFIX_0F7A:
+		emitbyte(0x0f);
+		emitbyte(0x7a);
+		break;
+	case X86ASM_PREFIX_0F7B:
+		emitbyte(0x0f);
+		emitbyte(0x7b);
+		break;
 	case X86ASM_PREFIX_DF:
 	case X86ASM_PREFIX_DE:
 	case X86ASM_PREFIX_DD:
@@ -549,7 +592,7 @@ bool x86asm::encode_insn(x86asm_insn *insn, x86opc_insn *opcode, int opcodeb, in
 	emitbyte(opcodeb);
 
 	/* encode the ops */
-	for (int i=0; i<3; i++) {
+	for (int i=0; i < 4; i++) {
 		if (!encode_op(&insn->op[i], &x86_op_type[opcode->op[i]], &esizes[i], eopsize, eaddrsize)) {
 			clearcode();
 			return false;
@@ -560,6 +603,16 @@ bool x86asm::encode_insn(x86asm_insn *insn, x86opc_insn *opcode, int opcodeb, in
 	if (modrmv != -1) emitbyte(modrmv);
 	if (sibv != -1) emitbyte(sibv);
 	
+	if (drexdest != -1) {
+		byte oc0 = 0;
+		if (drexoc0 != -1) {
+			oc0 = drexoc0;
+		}
+		byte drex = (drexdest << 4) | (oc0 << 3) | (rexprefix & 7);
+		rexprefix = 0;
+		emitbyte(drex);
+	}
+
 	if (disppos && addrsize == X86_ADDRSIZE64) {
 		// fix ip-relative disp in PM64 mode
 		dispsize = 4;
@@ -610,6 +663,103 @@ bool x86asm::encode_insn(x86asm_insn *insn, x86opc_insn *opcode, int opcodeb, in
 	return true;
 }
 
+bool x86asm::encode_vex_insn(x86asm_insn *insn, x86opc_vex_insn *opcode, int opcodeb, int additional_opcode, int eopsize, int eaddrsize)
+{
+	rexprefix = 0;
+	disppos = 0;
+	immsize = 0;
+	dispsize = 0;
+	vexvvvv = 0;
+	modrmv = -1;
+	sibv = -1;
+
+	if (additional_opcode != -1) {
+		emitmodrm_reg(additional_opcode);
+	}
+
+	switch (insn->lockprefix) {
+	case X86_PREFIX_LOCK: 
+		clearcode();
+		return false;
+	}
+	switch (insn->repprefix) {
+	case X86_PREFIX_REPNZ:
+	case X86_PREFIX_REPZ: 
+		clearcode();
+		return false;
+	}
+	switch (insn->segprefix) {
+	case X86_PREFIX_ES: emitbyte(0x26); break;
+	case X86_PREFIX_CS: emitbyte(0x2e); break;
+	case X86_PREFIX_SS: emitbyte(0x36); break;
+	case X86_PREFIX_DS: emitbyte(0x3e); break;
+	case X86_PREFIX_FS: emitbyte(0x64); break;
+	case X86_PREFIX_GS: emitbyte(0x65); break;
+	}
+
+	for (int i=0; i < 5; i++) {
+		if (!encode_op(&insn->op[i], &x86_op_type[opcode->op[i]], &esizes[i], eopsize, eaddrsize)) {
+			clearcode();
+			return false;
+		}
+	}
+
+	if ((opcode->vex & 0x30) == 0x10 // 0x0f-opcode
+	 && !(rexprefix & 3)          // no rexb/rexx
+	 && !(opcode->vex & W1)) {
+		// use short vex prefix
+		emitbyte(0xc5);
+		byte vex = (~rexprefix & 4) << 5
+			| ((~vexvvvv & 0xf) << 3)
+			| (opcode->vex & _256) >> 4 
+			| (opcode->vex & 0x3);
+		emitbyte(vex);
+	} else {
+		// use long vex prefix
+		emitbyte(0xc4);
+		byte vex = (~rexprefix & 7) << 5 | ((opcode->vex & 0x30) >> 4);
+		emitbyte(vex);
+		vex = (opcode->vex & W1) | ((~vexvvvv & 0xf) << 3) 
+			| (opcode->vex & _256) >> 4 
+			| (opcode->vex & 0x3);
+		emitbyte(vex);
+	}
+
+	emitbyte(opcodeb);
+
+	if (modrmv != -1) emitbyte(modrmv);
+	if (sibv != -1) emitbyte(sibv);
+
+	if (disppos && addrsize == X86_ADDRSIZE64) {
+		// fix ip-relative disp in PM64 mode
+		dispsize = 4;
+		disp -= address + code.size + dispsize + immsize;
+		if (simmsize(disp, 4) > 4) {
+			clearcode();
+			return false;
+		}
+	}
+	switch (dispsize) {
+	case 1:
+		emitbyte(disp);
+		break;
+	case 2:
+		emitword(disp);
+		break;
+	case 4:
+		emitdword(disp);
+		break;
+	}
+
+	switch (immsize) {
+	case 1:
+		emitbyte(imm);
+		break;
+	}
+
+	return true;
+}
+
 bool x86asm::encode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow_mem, int eopsize, int eaddrsize)
 {
 	switch (op->type) {
@@ -672,6 +822,11 @@ bool x86asm::encode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow
 		emitmodrm_mod(3);
 		emitmodrm_rm(op->xmm);
 		break;
+	case X86_OPTYPE_YMM:
+		if (!allow_reg) return false;
+		emitmodrm_mod(3);
+		emitmodrm_rm(op->ymm);
+		break;
 	default:
 		return false;
 	}
@@ -764,6 +919,16 @@ bool x86asm::encode_op(x86_insn_op *op, x86opc_insn_op *xop, int *esize, int eop
 	case TYPE_Ix:
 		/* fixed immediate */
 		return true;
+	case TYPE_I4: {
+		/* 4 bit immediate (see TYPE_VI, TYPE_YI) */
+		if (op->imm > 15) return false;
+		if (immsize == 0) {
+			immsize = 1;
+			imm = 0;
+		}
+		imm |= op->imm;
+		break;
+	}
 	case TYPE_J: {
 		/* relative branch offset */
 		int size = esizeop_ex(xop->size, eopsize);
@@ -836,27 +1001,94 @@ bool x86asm::encode_op(x86_insn_op *op, x86opc_insn_op *xop, int *esize, int eop
 	case TYPE_Sx:
 		/* extra picks segment register */
 		return true;
-	case TYPE_T:
-		/* reg of ModR/M picks test register */
-		emitmodrm_reg(op->trx);
-		if (op->trx > 7) rexprefix |= rexr;
+	case TYPE_V:
+		/* reg of ModR/M picks XMM register */
+		emitmodrm_reg(op->xmm);
+		if (op->xmm > 7) rexprefix |= rexr;
 		break;
 	case TYPE_VR:
 		/* rm of ModR/M picks XMM register */
 		emitmodrm_mod(3);
-		// fall throu
-	case TYPE_V:
-		/* reg of ModR/M picks XMM register */
-		emitmodrm_reg(op->xmm);
-		if (op->mmx > 7) rexprefix |= rexr;
+		emitmodrm_rm(op->xmm);
+		if (op->xmm > 7) rexprefix |= rexb;
 		break;
 	case TYPE_Vx:
 		break;
+	case TYPE_VV:
+		vexvvvv = op->xmm;
+		break;
+	case TYPE_VI: {
+		/* bits 7-4 of imm pick XMM register */
+		if (immsize == 0) {
+			immsize = 1;
+			imm = 0;
+		}
+		imm |= op->xmm << 4;
+		break;
+	}
+	case TYPE_VD:
+		if (drexdest == -1) {
+			drexdest = op->xmm;
+		} else {
+			if (drexdest != op->xmm) {
+				return false;
+			}
+		}
+		break;
+	case TYPE_VS:
+		if (op->type == X86_OPTYPE_XMM) {
+			if (drexoc0 == 0) {
+				emitmodrm_mod(3);
+				emitmodrm_rm(op->xmm);
+				if (op->xmm > 7) rexprefix |= rexb;				
+			} else {
+				emitmodrm_reg(op->xmm);
+				if (op->xmm > 7) rexprefix |= rexr;
+				if (drexoc0 == -1) drexoc0 = 0;
+			}
+		} else {
+			if (drexoc0 == 1) return false;
+			if (drexoc0 == -1) {
+				if (xop->info) return false;
+				drexoc0 = 1;
+			}
+			if (!encode_modrm(op, xop->size, true, true, eopsize, eaddrsize)) return false; //XXX
+			psize = esizeop(xop->size, eopsize); //XXX
+		}
+		break;
 	case TYPE_W:
 		/* ModR/M (XMM reg or memory) */
 		if (!encode_modrm(op, xop->size, true, true, eopsize, eaddrsize)) return false; //XXX
 		psize = esizeop(xop->size, eopsize); //XXX
 		break;
+	case TYPE_Y:
+		/* reg of ModR/M picks YMM register */
+		emitmodrm_reg(op->ymm);
+		if (op->ymm > 7) rexprefix |= rexr;
+		break;
+	case TYPE_YI: {
+		/* bits 7-4 of imm pick YMM register */
+		if (immsize == 0) {
+			immsize = 1;
+			imm = 0;
+		}
+		imm |= op->xmm << 4;
+		break;
+	}
+	case TYPE_YV:
+		vexvvvv = op->ymm;
+		break;
+	case TYPE_YR:
+		/* rm of ModR/M picks YMM register */
+		emitmodrm_mod(3);
+		emitmodrm_rm(op->ymm);
+		if (op->ymm > 7) rexprefix |= rexb;
+		break;
+	case TYPE_X:
+		/* ModR/M (YMM reg or memory) */
+		if (!encode_modrm(op, xop->size, true, true, eopsize, eaddrsize)) return false; //XXX
+		psize = esizeop(xop->size, eopsize); //XXX
+		break;
 	}
 	if (!psize) {
 //		set_error_msg(X86ASM_ERRMSG_INTERNAL"FIXME: size ??? %s, %d\n", __FILE__, __LINE__);
@@ -1082,6 +1314,8 @@ const char *x86asm::lsz2hsz(int size, int opsize)
 			return hsz64_16;
 		case 16:
 			return hsz128_16;
+		case 32:
+			return hsz256_16;
 		}
 	} else if (opsize == X86_OPSIZE32) {
 		switch (size) {
@@ -1097,6 +1331,8 @@ const char *x86asm::lsz2hsz(int size, int opsize)
 			return hsz64_32;
 		case 16:
 			return hsz128_32;
+		case 32:
+			return hsz256_32;
 		}
 	} else {
 		switch (size) {
@@ -1112,6 +1348,8 @@ const char *x86asm::lsz2hsz(int size, int opsize)
 			return hsz64_64;
 		case 16:
 			return hsz128_64;
+		case 32:
+			return hsz256_64;
 		}
 	}
 	return 0;
@@ -1124,17 +1362,15 @@ const char *x86asm::lsz2hsz(int size, int opsize)
 
 int x86asm::match_type(x86_insn_op *op, x86opc_insn_op *xop, int addrsize)
 {
-	const int *hop=lop2hop[op->type];
 	if (op->type == X86_OPTYPE_EMPTY && xop->type == TYPE_0) return MATCHTYPE_MATCH;
 	int r = MATCHTYPE_MATCH;
-	if (xop->type == TYPE_W && xop->extra == 1) {
-		r = MATCHTYPE_OPPREFIX;
-	} else if (op->type == X86_OPTYPE_MMX) {
+	if (op->type == X86_OPTYPE_MMX) {
 		if ((xop->type == TYPE_P || xop->type == TYPE_PR || xop->type == TYPE_Q)
 		&& (xop->size == SIZE_U || xop->size == SIZE_Z)) {
 			r = MATCHTYPE_NOOPPREFIX;
 		}
 	}
+	const byte *hop = lop2hop[op->type];
 	while (*hop) {
 		if (*hop == xop->type) {
 			if (xop->type == TYPE_Rx) {
@@ -1190,11 +1426,10 @@ bool x86asm::match_size(x86_insn_op *op, x86opc_insn_op *xop, int opsize)
 			hsz = immlsz2hsz(simmsize(op->imm - address - code.size - ssize, size), opsize);
 		} else {
 			hsz = immlsz2hsz(simmsize(op->imm, esizeop(xop->size, opsize)), opsize); //XXX
-//			hsz = immlsz2hsz(op->size, opsize);
 		}
-	} else if (op->type == X86_OPTYPE_MMX) {
-		return true;
-	} else if (op->type == X86_OPTYPE_XMM) {
+	} else if (op->type == X86_OPTYPE_YMM
+	 || op->type == X86_OPTYPE_XMM
+	 || op->type == X86_OPTYPE_MMX) {
 		return true;
 	} else {
 		hsz = lsz2hsz(op->size, opsize);
@@ -1209,17 +1444,17 @@ bool x86asm::match_size(x86_insn_op *op, x86opc_insn_op *xop, int opsize)
 	return false;
 }
 
-int x86asm::match_allops(x86asm_insn *insn, x86opc_insn *xinsn, int opsize, int addrsize)
+int x86asm::match_allops(x86asm_insn *insn, byte *xop, int maxop, int opsize, int addrsize)
 {
 	int m = 0;
-	for (int i = 0; i < 3; i++) {
-		int m2 = match_type(&insn->op[i], &x86_op_type[xinsn->op[i]], addrsize);
+	for (int i = 0; i < maxop; i++) {
+		int m2 = match_type(&insn->op[i], &x86_op_type[xop[i]], addrsize);
 		if (!m2 || (m && m != MATCHTYPE_MATCH && m2 != MATCHTYPE_MATCH && m != m2)) {
 			return MATCHTYPE_NOMATCH;
 		} else {
 			if (m2 > m) m = m2;
 		}
-		if (!match_size(&insn->op[i], &x86_op_type[xinsn->op[i]], opsize)) return MATCHTYPE_NOMATCH;
+		if (!match_size(&insn->op[i], &x86_op_type[xop[i]], opsize)) return MATCHTYPE_NOMATCH;
 	}
 	return m;
 }
@@ -1280,49 +1515,81 @@ void x86asm::match_opcode(x86opc_insn *opcode, x86asm_insn *insn, int prefix, by
 {
 	int n = match_opcode_name(insn->name, opcode->name, def_match);
 	namefound |= n;
-	if (n != MATCHOPNAME_NOMATCH) {
-		insn->opsizeprefix = X86_PREFIX_NO;
-		char opsizes[] = {X86_OPSIZE16, X86_OPSIZE32, X86_OPSIZE64};
-		char addrsizes[] = {X86_ADDRSIZE16, X86_ADDRSIZE32, X86_ADDRSIZE64};
+	if (n == MATCHOPNAME_NOMATCH) return;
+
+	insn->opsizeprefix = X86_PREFIX_NO;
+	char opsizes[] = {X86_OPSIZE16, X86_OPSIZE32, X86_OPSIZE64};
+	char addrsizes[] = {X86_ADDRSIZE16, X86_ADDRSIZE32, X86_ADDRSIZE64};
 		
-		switch (addrsize) {
-		case X86_ADDRSIZE32: swap(addrsizes[0], addrsizes[1]); break;
-		case X86_ADDRSIZE64: swap(addrsizes[0], addrsizes[2]); break;
-		case X86_ADDRSIZE16:
-		case X86_ADDRSIZEUNKNOWN:;
-		}
+	switch (addrsize) {
+	case X86_ADDRSIZE32: swap(addrsizes[0], addrsizes[1]); break;
+	case X86_ADDRSIZE64: swap(addrsizes[0], addrsizes[2]); break;
+	case X86_ADDRSIZE16:
+	case X86_ADDRSIZEUNKNOWN:;
+	}
 		
-		bool done1 = false;
-		/*
-		 * check all permutations of opsize and addrsize
-		 * if possible and necessary
-		 */
-		for (int o=0; o < 3; o++) {
-			switch (n) {
-			case MATCHOPNAME_MATCH_IF_OPSIZE16: done1 = true; break;
-			case MATCHOPNAME_MATCH_IF_OPSIZE32: o = 1; done1 = true; break;
-			case MATCHOPNAME_MATCH_IF_OPSIZE64: o = 2; done1 = true; break;
-			}
-			if (o == 2 && addrsize != X86_ADDRSIZE64) break;
-			bool done2 = !addrsize_depend;
-			for (int a=0; a < 2; a++) {
-				char as = addrsizes[a];
-		    		switch (n) {
-				case MATCHOPNAME_MATCH_IF_ADDRSIZE16: as = X86_ADDRSIZE16; done2 = true; break;
-				case MATCHOPNAME_MATCH_IF_ADDRSIZE32: as = X86_ADDRSIZE32; done2 = true; break;
-		    		case MATCHOPNAME_MATCH_IF_ADDRSIZE64: as = X86_ADDRSIZE64; done2 = true; break;
-				}
-				match_opcode_final(opcode, insn, prefix, opcodebyte, additional_opcode, opsizes[o], as, n);
-				if (done2) break;
-			}
-			if (done1) break;
+	bool done1 = false;
+	int o = 0;
+	/*
+	 * check all permutations of opsize and addrsize
+	 * if possible and necessary
+	 */
+	switch (n) {
+	case MATCHOPNAME_MATCH_IF_OPSIZE16: done1 = true; break;
+	case MATCHOPNAME_MATCH_IF_OPSIZE32: o = 1; done1 = true; break;
+	case MATCHOPNAME_MATCH_IF_OPSIZE64: o = 2; done1 = true; break;
+	}
+	for (; o < 3; o++) {
+		if (o == 2 && addrsize != X86_ADDRSIZE64) break;
+		switch (def_match) {
+		case MATCHOPNAME_MATCH_IF_OPPREFIX:
+			if (opsize == X86_OPSIZE16 && o == 0) continue;
+			if (opsize == X86_OPSIZE32 && o == 1) continue;
+			break;
+		case MATCHOPNAME_MATCH_IF_NOOPPREFIX:
+			if (opsize == X86_OPSIZE16 && o == 1) continue;
+			if (opsize == X86_OPSIZE32 && o == 0) continue;
+			break;
 		}
+		for (int a=0; a < 2; a++) {
+			char as = addrsizes[a];
+			bool done2 = false;
+	    		switch (n) {
+			case MATCHOPNAME_MATCH_IF_ADDRSIZE16: as = X86_ADDRSIZE16; done2 = true; break;
+			case MATCHOPNAME_MATCH_IF_ADDRSIZE32: as = X86_ADDRSIZE32; done2 = true; break;
+	    		case MATCHOPNAME_MATCH_IF_ADDRSIZE64: as = X86_ADDRSIZE64; done2 = true; break;
+			}
+			match_opcode_final(opcode, insn, prefix, opcodebyte, additional_opcode, opsizes[o], as, n);
+			if (done2) break;
+		}
+		if (done1) break;
+	}
+}
+
+void x86asm::match_vex_opcode(x86opc_vex_insn *opcode, x86asm_insn *insn, byte opcodebyte, int additional_opcode)
+{
+	int n = match_opcode_name(insn->name, opcode->name, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+	namefound |= n;
+	if (n == MATCHOPNAME_NOMATCH) return;
+
+	char addrsizes[] = {X86_ADDRSIZE16, X86_ADDRSIZE32, X86_ADDRSIZE64};
+		
+	switch (addrsize) {
+	case X86_ADDRSIZE32: swap(addrsizes[0], addrsizes[1]); break;
+	case X86_ADDRSIZE64: swap(addrsizes[0], addrsizes[2]); break;
+	case X86_ADDRSIZE16:
+	case X86_ADDRSIZEUNKNOWN:;
+	}
+
+	for (int a=0; a < 2; a++) {
+		char as = addrsizes[a];
+		match_vex_opcode_final(opcode, insn, opcodebyte, additional_opcode, opsize, as);
 	}
 }
 
 int x86asm::match_opcode_final(x86opc_insn *opcode, x86asm_insn *insn, int prefix, byte opcodebyte, int additional_opcode, int opsize, int addrsize, int match)
 {
-	switch (match_allops(insn, opcode, opsize, addrsize)) {
+	switch (match_allops(insn, opcode->op, 4, opsize, addrsize)) {
 	case MATCHTYPE_NOMATCH:
 		return false;
 	case MATCHTYPE_MATCH:
@@ -1341,7 +1608,6 @@ int x86asm::match_opcode_final(x86opc_insn *opcode, x86asm_insn *insn, int prefi
 		insn->opsizeprefix = X86_PREFIX_OPSIZE;
 		break;
 	}
-//	printf("o%ds%d: %02x (%d)\n", (opsize==X86_OPSIZE16) ? 16 : 32, (addrsize==X86_ADDRSIZE16) ? 16 : 32, opcodebyte, opcodebyte);
 	if (encode_insn(insn, opcode, opcodebyte, additional_opcode, prefix, opsize, addrsize)) {
 		pushcode();
 		newcode();
@@ -1349,6 +1615,18 @@ int x86asm::match_opcode_final(x86opc_insn *opcode, x86asm_insn *insn, int prefi
 	return true;
 }
 
+int x86asm::match_vex_opcode_final(x86opc_vex_insn *opcode, x86asm_insn *insn, byte opcodebyte, int additional_opcode, int opsize, int addrsize)
+{
+	if (match_allops(insn, opcode->op, 5, opsize, addrsize) == MATCHTYPE_NOMATCH) {
+		return false;
+	}
+	if (encode_vex_insn(insn, opcode, opcodebyte, additional_opcode, opsize, addrsize)) {
+		pushcode();
+		newcode();
+	}
+	return true;
+}
+
 void x86asm::match_opcodes(x86opc_insn *opcodes, x86asm_insn *insn, int prefix, int def_match)
 {
 	for (int i=0; i < 256; i++) {
@@ -1379,6 +1657,25 @@ void x86asm::match_opcodes(x86opc_insn *opcodes, x86asm_insn *insn, int prefix,
 	}
 }
 
+void x86asm::match_vex_opcodes(x86asm_insn *insn)
+{
+	for (int i=0; i < 256; i++) {
+		x86opc_vex_insn *opcodes = x86_vex_insns[i];
+		if (!opcodes) continue;
+		while (!opcodes->name && opcodes->op[0] == SPECIAL_TYPE_GROUP) {
+			for (int j=0; j < 8; j++) {
+				x86opc_vex_insn *group = &x86_group_vex_insns[opcodes->op[1]][j];
+				if (group->name) match_vex_opcode(group, insn, i, j);
+			}
+			opcodes++;
+		}
+		while (opcodes->name) {
+			match_vex_opcode(opcodes, insn, i, -1);
+			opcodes++;
+		}
+	}
+}
+
 void x86asm::match_fopcodes(x86asm_insn *insn)
 {
 	/* try modrm fopcodes */
@@ -1389,7 +1686,7 @@ void x86asm::match_fopcodes(x86asm_insn *insn)
 			if (n != MATCHOPNAME_NOMATCH) {
 				int eaddrsize = addrsize;
 				for (int k=0; k < 2; k++) {
-					if (match_allops(insn, &x86_modfloat_group_insns[i][j], opsize, eaddrsize)) {
+					if (match_allops(insn, x86_modfloat_group_insns[i][j].op, 4, opsize, eaddrsize)) {
 						if (encode_insn(insn, &x86_modfloat_group_insns[i][j], j<<3, -1, X86ASM_PREFIX_D8+i, opsize, eaddrsize)) {
 							pushcode();
 							newcode();
@@ -1407,11 +1704,11 @@ void x86asm::match_fopcodes(x86asm_insn *insn)
 	/* try the rest */
 	for (int i=0; i<8; i++) {
 		for (int j=0; j<8; j++) {
-			if (x86_float_group_insns[i][j].group==0) {
-				int n=match_opcode_name(insn->name, x86_float_group_insns[i][j].insn.name, MATCHOPNAME_MATCH);
+			if (x86_float_group_insns[i][j].group == 0) {
+				int n = match_opcode_name(insn->name, x86_float_group_insns[i][j].insn.name, MATCHOPNAME_MATCH);
 				namefound |= n;
 				if (n != MATCHOPNAME_NOMATCH) {
-					if (match_allops(insn, &x86_float_group_insns[i][j].insn, opsize, addrsize)) {
+					if (match_allops(insn, x86_float_group_insns[i][j].insn.op, 4, opsize, addrsize)) {
 						if (encode_insn(insn, &x86_float_group_insns[i][j].insn, 0xc0 | j<<3, -1, X86ASM_PREFIX_D8+i, opsize, addrsize)) {
 							pushcode();
 							newcode();
@@ -1421,13 +1718,13 @@ void x86asm::match_fopcodes(x86asm_insn *insn)
 				}
 			} else {
 				x86opc_insn *group=x86_float_group_insns[i][j].group;
-				for (int k=0; k<8; k++) {
+				for (int k=0; k < 8; k++) {
 					int n = match_opcode_name(insn->name, group[k].name, MATCHOPNAME_MATCH);
 					namefound |= n;
 					if (n != MATCHOPNAME_NOMATCH) {
 						int eaddrsize = addrsize;
 						for (int l=0; l < 2; l++) {
-							if (match_allops(insn, &group[k], opsize, eaddrsize)) {
+							if (match_allops(insn, group[k].op, 4, opsize, eaddrsize)) {
 								if (encode_insn(insn, &group[k], 0xc0 | j<<3 | k, -1, X86ASM_PREFIX_D8+i, opsize, eaddrsize)) {
 									pushcode();
 									newcode();
@@ -1487,6 +1784,19 @@ bool x86asm::opxmm(x86_insn_op *op, const char *xop)
 	}
 }
 
+bool x86asm::opymm(x86_insn_op *op, const char *xop)
+{
+	if (strlen(xop) == 4 && xop[0] == 'y' && xop[1] == 'm' && xop[2] == 'm'
+	 && xop[3] >= '0' && xop[3] <= '7') {
+		op->type = X86_OPTYPE_YMM;
+		op->size = 32;
+		op->xmm = xop[3] - '0';
+		return true;
+	} else {
+		return false;
+	}
+}
+
 bool x86asm::opseg(x86_insn_op *op, const char *xop)
 {
 	for (int i=0; i<8; i++) {
@@ -1569,8 +1879,8 @@ bool x86asm::opmem(x86asm_insn *asm_insn, x86_insn_op *op, const char *s)
 
 	tok(&s, token, sizeof token, sep);
 	
-	static const char *types[] = {"byte", "word", "dword", "pword", "qword", "oword", "single", "double", "extended"};
-	static byte type_size[] = {1, 2, 4, 6, 8, 16, 4, 8, 10};
+	static const char *types[] = {"byte", "word", "dword", "pword", "qword", "oword", "ymmword", "single", "double", "extended"};
+	static byte type_size[] = {1, 2, 4, 6, 8, 16, 32, 4, 8, 10};
 	// typecast
 	for (uint i=0; i < sizeof types / sizeof types[0]; i++) {
 		if (strcmp(token, types[i]) == 0) {
@@ -1609,6 +1919,7 @@ bool x86asm::opmem(x86asm_insn *asm_insn, x86_insn_op *op, const char *s)
 	bool need_rex = false;
 
 	int sign = 1;
+	sep = "[]()*+-";
 	while (1) {
 cont:
 		tok(&s, token, sizeof token, sep);
@@ -1748,11 +2059,6 @@ bool x86asm::opspecialregs(x86_insn_op *op, const char *xop)
 		op->size = 4;
 		op->drx = w;
 		return true;
-	} else if (ht_strncmp(xop, "tr", 2) == 0) {
-		op->type = X86_OPTYPE_TRX;
-		op->size = 4;
-		op->trx = w;
-		return true;
 	}
 	return false;
 }
@@ -1760,11 +2066,13 @@ bool x86asm::opspecialregs(x86_insn_op *op, const char *xop)
 bool x86asm::translate_str(asm_insn *asm_insn, const char *s)
 {
 	x86asm_insn *insn=(x86asm_insn*)asm_insn;
-	char *opp[3], op[3][256];
+	char *opp[5], op[5][256];
 	opp[0]=op[0];
 	opp[1]=op[1];
 	opp[2]=op[2];
-	for (int i=0; i<3; i++) {
+	opp[3]=op[3];
+	opp[4]=op[4];
+	for (int i=0; i<5; i++) {
 		insn->op[i].need_rex = insn->op[i].forbid_rex = false;
 		insn->op[i].type = X86_OPTYPE_EMPTY;
 	}
@@ -1796,13 +2104,14 @@ bool x86asm::translate_str(asm_insn *asm_insn, const char *s)
 	/**/
 	splitstr(s, insn->n, sizeof insn->n, (char**)&opp, 256);
 	insn->name=insn->n;
-	for (int i=0; i<3; i++) {
+	for (int i=0; i<5; i++) {
 		if (!*op[i]) break;
 
 		if (!(opplugimm(&insn->op[i], op[i])
 		 || opreg(&insn->op[i], op[i])
 		 || opmmx(&insn->op[i], op[i])
 		 || opxmm(&insn->op[i], op[i])
+		 || opymm(&insn->op[i], op[i])
 		 || opfarptr(&insn->op[i], op[i])
 		 || opimm(&insn->op[i], op[i])
 		 || opseg(&insn->op[i], op[i])
@@ -1834,7 +2143,7 @@ int x86asm::simmsize(uint64 imm, int immsize)
 	return 8;
 }
 
-void x86asm::splitstr(const char *s, char *name, int size, char *op[3], int opsize)
+void x86asm::splitstr(const char *s, char *name, int size, char *op[5], int opsize)
 {
 	const char *a, *b;
 	bool wantbreak = false;
@@ -1842,6 +2151,8 @@ void x86asm::splitstr(const char *s, char *name, int size, char *op[3], int opsi
 	*op[0]=0;
 	*op[1]=0;
 	*op[2]=0;
+	*op[3]=0;
+	*op[4]=0;
 	/* find name */
 	whitespaces(s);
 	a = s;
@@ -1849,7 +2160,7 @@ void x86asm::splitstr(const char *s, char *name, int size, char *op[3], int opsi
 	b = s;
 	ht_strlcpy(name, a, MIN(b-a+1, size));
 	/* find ops */
-	for (int i = 0; i < 3; i++) {
+	for (int i = 0; i < 5; i++) {
 		whitespaces(s);
 		if (!*s) break;
 		a = s;
@@ -1922,7 +2233,7 @@ bool x86_64asm::opreg(x86_insn_op *op, const char *xop)
 				op->type = X86_OPTYPE_REG;
 				op->size = reg2size[i];
 				op->reg = j;
-				if (j > 7 /*|| i == 3*/ || (i == 0 && j > 3)) {
+				if (j > 7 || (i == 0 && j > 3)) {
 					op->need_rex = true;
 				}
 				return true;
@@ -1963,3 +2274,25 @@ bool x86_64asm::opxmm(x86_insn_op *op, const char *xop)
 		return false;
 	}
 }
+
+bool x86_64asm::opymm(x86_insn_op *op, const char *xop)
+{
+	int slen = strlen(xop);
+	if ((slen == 4 || slen == 5) && xop[0] == 'y' && xop[1] == 'm' && xop[2] == 'm'
+	 && xop[3] >= '0' && xop[3] <= '9') {
+		int x = xop[3] - '0';
+		if (slen == 5) {
+			if (xop[4] < '0' || xop[4] > '9') return false;
+			x *= 10;
+			x += xop[4] - '0';
+			if (x > 15) return false;
+		}
+		op->type = X86_OPTYPE_YMM;
+		op->size = 32;
+		op->xmm = x;
+		if (x > 7) op->need_rex = true;
+		return true;
+	} else {
+		return false;
+	}
+}
