@@ -3,7 +3,7 @@
  *	x86dis.cc
  *
  *	Copyright (C) 1999-2002 Stefan Weyergraf
- *	Copyright (C) 2005-2006 Sebastian Biallas (sb@biallas.net)
+ *	Copyright (C) 2005-2008 Sebastian Biallas (sb@biallas.net)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -38,6 +38,19 @@
 #define rexx(rex) ((rex)&0x02)
 #define rexb(rex) ((rex)&0x01)
 
+#define drexdest(drex) ((drex)>>4)
+#define oc0(drex) (!!((drex)&8))
+
+#define vexw(vex) (!!((vex)&0x80))
+#define vexr(vex) (!((vex)&0x80))
+#define vexx(vex) (!((vex)&0x40))
+#define vexb(vex) (!((vex)&0x20))
+
+#define vexl(vex) (!!((vex)&0x04))
+#define vexmmmm(vex) ((vex)&0xf)
+#define vexvvvv(vex) (((~(vex))>>3)&0xf)
+#define vexpp(vex) ((vex)&0x3)
+
 static int modrm16_1[8] = { X86_REG_BX, X86_REG_BX, X86_REG_BP, X86_REG_BP,
                      X86_REG_SI, X86_REG_DI, X86_REG_BP, X86_REG_BX};
 static int modrm16_2[8] = { X86_REG_SI, X86_REG_DI, X86_REG_SI, X86_REG_DI,
@@ -70,6 +83,9 @@ dis_insn *x86dis::decode(byte *code, int Maxlen, CPU_ADDR Addr)
 	addr = Addr;
 	modrm = -1;
 	sib = -1;
+	drex = -1;
+	special_imm = -1;
+	have_disp = false;
 	memset(&insn, 0, sizeof insn);
 	insn.invalid = false;
 	insn.eopsize = opsize;
@@ -92,7 +108,7 @@ dis_insn *x86dis::decode(byte *code, int Maxlen, CPU_ADDR Addr)
 		insn.repprefix = X86_PREFIX_NO;
 		insn.segprefix = X86_PREFIX_NO;
 		insn.rexprefix = 0;
-		for (int i = 1; i < 3; i++) insn.op[i].type = X86_OPTYPE_EMPTY;
+		for (int i = 1; i < 5; i++) insn.op[i].type = X86_OPTYPE_EMPTY;
 	} else {
 		insn.size = codep - ocodep;
 		if (fixdisp) {
@@ -137,9 +153,10 @@ static bool is_xmm_op(x86dis_insn *insn, char size)
 	}
 }
 
-void x86dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow_mem, bool mmx, bool xmm)
+void x86dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow_mem, bool mmx, bool xmm, bool ymm)
 {
 	int modrm = getmodrm();
+	getdisp();
 	int mod = mkmod(modrm);
 	int rm = mkrm(modrm);
 	if (mod == 3) {
@@ -154,6 +171,9 @@ void x86dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow
 		} else if (mmx) {
 			op->type = X86_OPTYPE_MMX;
 			op->mmx = rm;
+		} else if (ymm) {
+			op->type = X86_OPTYPE_YMM;
+			op->xmm = rm;
 		} else {
 			op->type = X86_OPTYPE_REG;
 			op->reg = rm;
@@ -174,7 +194,7 @@ void x86dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow
 		if (insn.eaddrsize == X86_ADDRSIZE16) {
 			if (mod == 0 && rm == 6) {
 				op->mem.hasdisp = true;
-				op->mem.disp = getword();
+				op->mem.disp = disp;
 				op->mem.base = X86_REG_NO;
 				op->mem.index = X86_REG_NO;
 				op->mem.scale = 0;
@@ -189,18 +209,18 @@ void x86dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow
 					break;
 				case 1:
 					op->mem.hasdisp = true;
-					op->mem.disp = sint64(sint8(getbyte()));
+					op->mem.disp = sint64(sint8(disp));
 					break;
 				case 2:
 					op->mem.hasdisp = true;
-					op->mem.disp = sint64(sint16(getword()));
+					op->mem.disp = sint64(sint16(disp));
 					break;
 				}
 			}
 		} else {
 			if (mod == 0 && rm == 5) {
 				op->mem.hasdisp = true;
-				op->mem.disp = getdword();
+				op->mem.disp = disp;
 				op->mem.base = X86_REG_NO;
 				op->mem.index = X86_REG_NO;
 				op->mem.scale = 0;
@@ -217,11 +237,11 @@ void x86dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow
 					break;
 				case 1:
 					op->mem.hasdisp = true;
-					op->mem.disp = sint64(sint8(getbyte()));
+					op->mem.disp = sint64(sint8(disp));
 					break;
 				case 2:
 					op->mem.hasdisp = true;
-					op->mem.disp = sint64(sint32(getdword()));
+					op->mem.disp = sint64(sint32(disp));
 					break;
 				}
 			}
@@ -229,6 +249,91 @@ void x86dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow
 	}
 }
 
+void x86dis::decode_sib(x86_insn_op *op, int mod)
+{
+	int sib = getsib();
+	int scale = mkscale(sib);
+	int index = mkindex(sib);
+	int base = mkbase(sib);
+	int sdisp = mod;
+	if ((base & 0x7) == 5 && mod == 0) {
+		base = X86_REG_NO;
+		sdisp = 2;
+	}
+	if (index == 4) {
+		index = X86_REG_NO;
+	}
+	op->mem.base = base;
+	op->mem.index = index;
+	op->mem.scale = sibscale[scale];
+	switch (sdisp) {
+	case 0:
+		op->mem.hasdisp = false;
+		op->mem.disp = 0;
+		break;
+	case 1:
+		op->mem.hasdisp = true;
+		op->mem.disp = sint64(sint8(disp));
+		break;
+	case 2:
+		op->mem.hasdisp = true;
+		op->mem.disp = sint64(sint32(disp));
+		break;
+	}
+}
+
+void x86dis::decode_vex_insn(x86opc_vex_insn *xinsn)
+{
+	if (xinsn) {
+		byte vex = (insn.vexprefix.w << 7) | (insn.vexprefix.l << 6)
+			| (insn.vexprefix.mmmm << 4) | insn.vexprefix.pp;
+		while (!xinsn->name && xinsn->op[0] == SPECIAL_TYPE_GROUP) {
+			if (xinsn->vex == vex) {
+				getdisp();
+				int m = mkreg(getmodrm()) & 0x7;
+				xinsn = &x86_group_vex_insns[xinsn->op[1]][m];
+				if (!xinsn->name) {
+					invalidate();
+				} else {
+					insn.name = xinsn->name;
+					for (int i = 0; i < 5; i++) {
+						decode_op(&insn.op[i], &x86_op_type[xinsn->op[i]]);
+					}
+					return;
+				}
+			}
+			xinsn++;
+		}
+		while (xinsn->name) {
+			if (xinsn->vex == vex) {
+				insn.name = xinsn->name;
+				
+				for (int i = 0; i < 5; i++) {
+					x86opc_insn_op *op = &x86_op_type[xinsn->op[i]];
+					switch (op->type) {
+					case TYPE_E:
+					case TYPE_M:
+					case TYPE_W:
+					case TYPE_X:
+						/* get whole modrm/sib/disp stuff first
+		    				 * (otherwise a TYPE_VI immediate might 
+						 * get fetched fetched before the modrm stuff)
+						 */
+						getdisp();
+					}
+				}
+				
+				for (int i = 0; i < 5; i++) {
+					decode_op(&insn.op[i], &x86_op_type[xinsn->op[i]]);
+				}
+				return;
+			}
+			xinsn++;
+		}
+	}
+	invalidate();
+}
+
 void x86dis::decode_insn(x86opc_insn *xinsn)
 {
 	if (!xinsn->name) {
@@ -273,7 +378,53 @@ void x86dis::decode_insn(x86opc_insn *xinsn)
 					}
 					break;
 				}
-				// fall throu
+				invalidate();
+				break;
+			case 0xc4:
+			case 0xc5: {
+				if (insn.opsizeprefix != X86_PREFIX_NO
+				 || insn.lockprefix != X86_PREFIX_NO
+				 || insn.repprefix != X86_PREFIX_NO
+				 || insn.rexprefix != 0) {
+					invalidate();
+					break;
+				}
+				byte vex = getbyte();
+				if (addrsize != X86_ADDRSIZE64
+				 && (vex & 0xc0) != 0xc0) {
+					modrm = vex;
+					decode_insn(c == 0xc4 ? &x86_les : &x86_lds);
+					break;
+				}
+				insn.rexprefix = 0x40;
+				insn.rexprefix = vexr(vex) << 2;
+				if (c == 0xc4) {
+					// 3 byte vex
+					insn.rexprefix |= vexx(vex) << 1;
+					insn.rexprefix |= vexb(vex);
+					insn.vexprefix.mmmm = vexmmmm(vex);
+					if (insn.vexprefix.mmmm == 0
+					 || insn.vexprefix.mmmm > 3) {
+						invalidate();
+						break;
+					}
+					vex = getbyte();
+					insn.vexprefix.w |= vexw(vex);
+				} else {
+					// 2 byte vex
+					insn.vexprefix.mmmm = 1;
+				}
+				insn.vexprefix.vvvv = vexvvvv(vex);
+				insn.vexprefix.l = vexl(vex);
+				insn.vexprefix.pp = vexpp(vex);
+				if (addrsize != X86_ADDRSIZE64) {
+					insn.rexprefix = 0;
+				}
+				
+				insn.opcode = getbyte();
+				decode_vex_insn(x86_vex_insns[insn.opcode]);
+				break;
+			}
 			default:
 				invalidate();
 				break;
@@ -320,7 +471,7 @@ void x86dis::decode_insn(x86opc_insn *xinsn)
 		checkInfo(xinsn);
 		
 		insn.name = xinsn->name;
-		for (int i = 0; i < 3; i++) {
+		for (int i = 0; i < 4; i++) {
 			decode_op(&insn.op[i], &x86_op_type[xinsn->op[i]]);
 		}
 	}
@@ -363,7 +514,7 @@ void x86dis::decode_op(x86_insn_op *op, x86opc_insn_op *xop)
 	}
 	case TYPE_E: {
 		/* ModR/M (general reg or memory) */
-		decode_modrm(op, xop->size, (xop->size != SIZE_P), true, false, false);
+		decode_modrm(op, xop->size, (xop->size != SIZE_P), true, false, false, false);
 		break;
 	}
 	case TYPE_F: {
@@ -458,6 +609,13 @@ void x86dis::decode_op(x86_insn_op *op, x86opc_insn_op *xop)
 		op->imm = xop->extra;
 		break;
 	}
+	case TYPE_I4: {
+		/* 4 bit immediate (see TYPE_VI, TYPE_YI) */
+		op->type = X86_OPTYPE_IMM;
+		op->size = 1;
+		op->imm = getspecialimm() & 0xf;
+		break;
+	}
 	case TYPE_J: {
 		/* relative branch offset */
 		op->type = X86_OPTYPE_IMM;
@@ -491,7 +649,7 @@ void x86dis::decode_op(x86_insn_op *op, x86opc_insn_op *xop)
 	}
 	case TYPE_M: {
 		/* ModR/M (memory only) */
-		decode_modrm(op, xop->size, false, true, false, false);
+		decode_modrm(op, xop->size, false, true, false, false, false);
 		break;
 	}
 	case TYPE_MR: {
@@ -502,7 +660,7 @@ void x86dis::decode_op(x86_insn_op *op, x86opc_insn_op *xop)
 		if (mod == 3) {
 			xopsize = xop->extra;
 		}
-		decode_modrm(op, xopsize, (xopsize != SIZE_P), true, false, false);
+		decode_modrm(op, xopsize, (xopsize != SIZE_P), true, false, false, false);
 		break;
 	}
 	case TYPE_O: {
@@ -560,7 +718,7 @@ void x86dis::decode_op(x86_insn_op *op, x86opc_insn_op *xop)
 	}
 	case TYPE_Q: {
 		/* ModR/M (MMX reg or memory) */
-		decode_modrm(op, xop->size, true, true, true, false);
+		decode_modrm(op, xop->size, true, true, true, false, false);
 		break;
 	}
 	case TYPE_R: {
@@ -600,13 +758,6 @@ void x86dis::decode_op(x86_insn_op *op, x86opc_insn_op *xop)
 		if (op->seg > 5) invalidate();
 		break;
 	}
-	case TYPE_T: {
-		/* reg of ModR/M picks test register */
-		op->type = X86_OPTYPE_TRX;
-		op->size = esizeop(xop->size);
-		op->trx = mkreg(getmodrm());
-		break;
-	}
 	case TYPE_V: {
 		/* reg of ModR/M picks XMM register */
 		op->type = X86_OPTYPE_XMM;
@@ -614,6 +765,20 @@ void x86dis::decode_op(x86_insn_op *op, x86opc_insn_op *xop)
 		op->xmm = mkreg(getmodrm());
 		break;
 	}
+	case TYPE_VI: {
+		/* bits 7-4 of imm picks XMM register */
+		op->type = X86_OPTYPE_XMM;
+		op->size = 16;
+		op->xmm = getspecialimm() >> 4;
+		break;
+	}
+	case TYPE_VV: {
+		/* VEX.vvvv picks XMM register */
+		op->type = X86_OPTYPE_XMM;
+		op->size = 16;
+		op->xmm = insn.vexprefix.vvvv;
+		break;
+	}
 	case TYPE_Vx: {
 		/* extra picks XMM register */
 		op->type = X86_OPTYPE_XMM;
@@ -633,40 +798,47 @@ void x86dis::decode_op(x86_insn_op *op, x86opc_insn_op *xop)
 		break;
 	case TYPE_W:
 		/* ModR/M (XMM reg or memory) */
-		decode_modrm(op, xop->size, true, true, false, true);
+		decode_modrm(op, xop->size, true, true, false, true, false);
+		break;
+	case TYPE_VD:
+		op->type = X86_OPTYPE_XMM;
+		op->size = 16;
+		op->reg = drexdest(getdrex());
+		break;
+	case TYPE_VS: 
+		if (xop->info && oc0(getdrex())) {
+			invalidate();
+		}
+		if (oc0(getdrex()) ^ xop->extra) {
+			decode_modrm(op, xop->size, true, true, false, true, false);
+		} else {
+			op->type = X86_OPTYPE_XMM;
+			op->size = 16;
+			op->xmm = mkreg(getmodrm());
+		}
+		break;
+	case TYPE_Y:
+		/* reg of ModR/M picks XMM register */
+		op->type = X86_OPTYPE_YMM;
+		op->size = 32;
+		op->ymm = mkreg(getmodrm());
+		break;
+	case TYPE_YV: {
+		/* VEX.vvvv picks YMM register */
+		op->type = X86_OPTYPE_YMM;
+		op->size = 32;
+		op->ymm = insn.vexprefix.vvvv;
 		break;
 	}
-}
-
-void x86dis::decode_sib(x86_insn_op *op, int mod)
-{
-	int sib = getsib();
-	int scale = mkscale(sib);
-	int index = mkindex(sib);
-	int base = mkbase(sib);
-	int disp = mod;
-	if ((base & 0x7) == 5 && mod == 0) {
-		base = X86_REG_NO;
-		disp = 2;
-	}
-	if (index == 4) {
-		index = X86_REG_NO;
-	}
-	op->mem.base = base;
-	op->mem.index = index;
-	op->mem.scale = sibscale[scale];
-	switch (disp) {
-	case 0:
-		op->mem.hasdisp = false;
-		op->mem.disp = 0;
+	case TYPE_YI:
+		/* bits 7-4 of imm picks YMM register */
+		op->type = X86_OPTYPE_YMM;
+		op->size = 32;
+		op->ymm = getspecialimm() >> 4;
 		break;
-	case 1:
-		op->mem.hasdisp = true;
-		op->mem.disp = sint64(sint8(getbyte()));
-		break;
-	case 2:
-		op->mem.hasdisp = true;
-		op->mem.disp = sint64(sint32(getdword()));
+	case TYPE_X:
+		/* ModR/M (XMM reg or memory) */
+		decode_modrm(op, xop->size, true, true, false, false, true);
 		break;
 	}
 }
@@ -693,6 +865,8 @@ int x86dis::esizeop(uint c)
 		return 8;
 	case SIZE_O:
 		return 16;
+	case SIZE_Y:
+		return 32;
 	case SIZE_T:
 		return 10;
 	case SIZE_V:
@@ -801,12 +975,90 @@ void x86dis::filloffset(CPU_ADDR &addr, uint64 offset)
 	addr.addr32.offset = offset;
 }
 
+uint32 x86dis::getdisp()
+{
+	if (have_disp) return disp;
+	have_disp = true;
+	int modrm = getmodrm();
+	int mod = mkmod(modrm);
+	if (mod == 3) return 0;
+	int rm = mkrm(modrm);
+	if (insn.eaddrsize == X86_ADDRSIZE16) {
+		if (mod == 0 && rm == 6) {
+			disp = getword();
+		} else {
+			switch (mod) {
+			case 1:
+				disp = getbyte();
+				break;
+			case 2:
+				disp = getword();
+				break;
+			}
+		}
+	} else {
+		if (mod == 0 && rm == 5) {
+			disp = getdword();
+		} else if (rm == 4) {
+			int sib = getsib();
+			int base = mkbase(sib);
+			int d = mod;
+			if ((base & 0x7) == 5 && mod == 0) {
+				d = 2;
+			}
+			switch (d) {
+			case 1:
+				disp = getbyte();
+				break;
+			case 2:
+				disp = getdword();
+				break;
+			}
+		} else {
+			switch (mod) {
+			case 1:
+				disp = getbyte();
+				break;
+			case 2:
+				disp = getdword();
+				break;
+			}
+		}
+	}
+	return disp;
+}
+
 int x86dis::getmodrm()
 {
 	if (modrm == -1) modrm = getbyte();
 	return modrm;
 }
 
+int x86dis::getdrex()
+{
+	if (drex == -1) {
+		getmodrm();
+		int modrm = getmodrm();
+		int mod = mkmod(modrm);
+		int rm = mkrm(modrm);
+		if (mod != 3 && (rm & 7) == 4) {
+			getsib();
+		}
+		drex = getbyte();
+		if (addrsize != X86_ADDRSIZE64) {
+			drex &= 0x78;
+		}
+		insn.rexprefix = drex & 0x7;
+	}
+	return drex;
+}
+
+int x86dis::getspecialimm()
+{
+	if (special_imm == -1) special_imm = getbyte();
+	return special_imm;
+}
+
 const char *x86dis::getName()
 {
 	return "x86/Disassembler";
@@ -912,44 +1164,6 @@ void x86dis::prefixes()
 	}
 }
 
-#if 0
-bool x86dis::special_param_ambiguity(x86opc_insn *insn)
-{
-	if (insn->name) {
-		return insn->name[0] == '~';
-	} else {
-		return false;
-	}
-	bool regc = false;
-	bool memc = false;
-	bool segc = false;
-
-	for (int i=0; i<3; i++) {
-		switch (disasm_insn->op[i].type) {
-		case X86_OPTYPE_SEG:
-			segc = true;
-			break;
-		case X86_OPTYPE_REG:
-		case X86_OPTYPE_CRX:
-		case X86_OPTYPE_DRX:
-		case X86_OPTYPE_TRX:
-		case X86_OPTYPE_STX:
-/*		case X86_OPTYPE_MMX:
-		case X86_OPTYPE_XMM:*/
-			regc = true;
-			break;
-		case X86_OPTYPE_MEM:
-			memc = true;
-			break;
-		}
-	}
-	return (memc && !regc)
-		|| (memc && segc)
-		|| (strcmp(disasm_insn->name, "movzx") == 0)
-		|| (strncmp(disasm_insn->name, "movsx", 5) == 0);
-}
-#endif
-
 static const char *regs(x86dis_insn *insn, int mode, int nr)
 {
 	if (insn->rexprefix) {
@@ -1024,9 +1238,6 @@ void x86dis::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *
 	case X86_OPTYPE_DRX:
 		sprintf(opstr, "dr%d", op->drx);
 		break;
-	case X86_OPTYPE_TRX:
-		sprintf(opstr, "tr%d", op->trx);
-		break;
 	case X86_OPTYPE_STX:
 		if (op->stx) {
 			sprintf(opstr, "st%s(%s%d%s)%s", 
@@ -1042,6 +1253,9 @@ void x86dis::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *
 	case X86_OPTYPE_XMM:
 		sprintf(opstr, "xmm%d", op->xmm);
 		break;
+	case X86_OPTYPE_YMM:
+		sprintf(opstr, "ymm%d", op->ymm);
+		break;
 	case X86_OPTYPE_MEM: {
 		char *d=opstr;
 		if (explicit_params) {
@@ -1084,6 +1298,9 @@ void x86dis::str_op(char *opstr, int *opstrlen, x86dis_insn *insn, x86_insn_op *
 				case 16:
 					d += sprintf(d, "oword ptr ");
 					break;
+				case 32:
+					d += sprintf(d, "ymmword ptr ");
+					break;
 				}
 			}
 		}
@@ -1259,6 +1476,14 @@ void x86dis::str_format(char **str, const char **format, char *p, char *n, char
 					t=op[2];
 					tl=oplen[2];
 					break;
+				case DISASM_STRF_FORTH:
+					t=op[3];
+					tl=oplen[3];
+					break;
+				case DISASM_STRF_FIFTH:
+					t=op[4];
+					tl=oplen[4];
+					break;
 				}
 				if (tl) {
 					memcpy(s, t, tl);
@@ -1271,7 +1496,7 @@ void x86dis::str_format(char **str, const char **format, char *p, char *n, char
 			}
 			break;
 		case DISASM_STRF_COND: {
-			char *t=0;
+			char *t = NULL;
 			f++;
 			switch (*f) {
 			case DISASM_STRF_PREFIX:
@@ -1289,6 +1514,12 @@ void x86dis::str_format(char **str, const char **format, char *p, char *n, char
 			case DISASM_STRF_THIRD:
 				t=op[2];
 				break;
+			case DISASM_STRF_FORTH:
+				t=op[3];
+				break;
+			case DISASM_STRF_FIFTH:
+				t=op[4];
+				break;
 			}
 			f += 2;
 			if (t && t[0]) {
@@ -1352,12 +1583,12 @@ const char *x86dis::strf(dis_insn *disasm_insn, int opt, const char *format)
 	const char *iname = insn->name;
 	bool explicit_params = (options & X86DIS_STYLE_EXPLICIT_MEMSIZE) || iname[0] == '~';
 
-	char ops[3][512];	/* FIXME: possible buffer overflow ! */
-	char *op[3];
-	int oplen[3];
+	char ops[5][512];	/* FIXME: possible buffer overflow ! */
+	char *op[5];
+	int oplen[5];
 
 	if (options & DIS_STYLE_HIGHLIGHT) enable_highlighting();
-	for (int i=0; i < 3; i++) {
+	for (int i=0; i < 5; i++) {
 		op[i] = (char*)&ops[i];
 		str_op(op[i], &oplen[i], insn, &insn->op[i], explicit_params);
 	}
@@ -1422,8 +1653,6 @@ bool x86dis::validInsn(dis_insn *disasm_insn)
  */
 
 x86opc_insn (*x86_64dis::x86_64_insns)[256];
-x86opc_insn (*x86_64dis::x86_64_insns_ext)[256];
-x86opc_insn (*x86_64dis::x86_64_group_insns)[X86_GROUPS][8];
 
 x86_64dis::x86_64dis()
 	: x86dis(X86_OPSIZE32, X86_ADDRSIZE64)
@@ -1447,9 +1676,10 @@ void x86_64dis::prepInsns()
 	x86_insns = x86_64_insns;
 }
 
-void x86_64dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow_mem, bool mmx, bool xmm)
+void x86_64dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow_mem, bool mmx, bool xmm, bool ymm)
 {
 	int modrm = getmodrm();
+	getdisp();
 	int mod = mkmod(modrm);
 	int rm = mkrm(modrm);
 	if (mod == 3) {
@@ -1463,6 +1693,9 @@ void x86_64dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool al
 		} else if (mmx) {
 			op->type = X86_OPTYPE_MMX;
 			op->mmx = rm & 0x7; // no rex-extension
+		} else if (ymm) {
+			op->type = X86_OPTYPE_YMM;
+			op->mmx = rm;
 		} else {
 			op->type = X86_OPTYPE_REG;
 			op->reg = rm;
@@ -1481,7 +1714,7 @@ void x86_64dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool al
 
 		if (mod == 0 && (rm & 0x7) == 5) {
 			op->mem.hasdisp = true;
-			op->mem.disp = sint32(getdword());
+			op->mem.disp = sint32(disp);
 			fixdisp = true;
 //			op->mem.base = X86_REG_IP;
 			op->mem.base = X86_REG_NO;
@@ -1500,11 +1733,11 @@ void x86_64dis::decode_modrm(x86_insn_op *op, char size, bool allow_reg, bool al
 				break;
 			case 1:
 				op->mem.hasdisp = true;
-				op->mem.disp = sint64(sint8(getbyte()));
+				op->mem.disp = sint64(sint8(disp));
 				break;
 			case 2:
 				op->mem.hasdisp = true;
-				op->mem.disp = sint64(sint32(getdword()));
+				op->mem.disp = sint64(sint32(disp));
 				break;
 			}
 		}
