@@ -95,7 +95,9 @@ x86opc_insn_op x86_op_type[] = {
 {TYPE_M, 0, 0, SIZE_P},
 #define Mq	Mp+1
 {TYPE_M, 0, 0, SIZE_Q},
-#define Mr	Mq+1
+#define Mv	Mq+1
+{TYPE_M, 0, 0, SIZE_V},
+#define Mr	Mv+1
 {TYPE_M, 0, 0, SIZE_R},
 #define Mo	Mr+1
 {TYPE_M, 0, 0, SIZE_O},
@@ -1725,7 +1727,9 @@ x86opc_insn x86_opc_group_insns[X86_OPC_GROUPS][256] = {
 /* e8 */
 {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0},
 /* f0 */
-{0}, {0}, {0}, {0}, {0}, {0}, {0}, {0},
+{"movbe", {Gv, Mv}},
+{"movbe", {Mv, Gv}},
+{0}, {0}, {0}, {0}, {0}, {0},
 /* f8 */
 {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0},
 },
@@ -1827,7 +1831,9 @@ x86opc_insn x86_opc_group_insns[X86_OPC_GROUPS][256] = {
 /* 78 */
 {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0},
 /* 80 */
-{0}, {0}, {0}, {0}, {0}, {0}, {0}, {0},
+{"invept", {Gr, Mo}},
+{"invvpid", {Gr, Mo}},
+{0}, {0}, {0}, {0}, {0}, {0},
 /* 88 */
 {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0},
 /* 90 */
@@ -1862,7 +1868,9 @@ x86opc_insn x86_opc_group_insns[X86_OPC_GROUPS][256] = {
 /* e8 */
 {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0},
 /* f0 */
-{0}, {0}, {0}, {0}, {0}, {0}, {0}, {0},
+{"movbe", {Gv, Mv}},
+{"movbe", {Mv, Gv}},
+{0}, {0}, {0}, {0}, {0}, {0},
 /* f8 */
 {0}, {0}, {0}, {0}, {0}, {0}, {0}, {0},
 },
