@@ -52,13 +52,25 @@ static uint32 hexd(const char *s)
 }
 //
 
-TAGSTRING *tag_make_sel(TAGSTRING *buf, const char *string)
+static TAGSTRING *tag_error(TAGSTRING *buf, int maxlen)
 {
-	return tag_make_ref(buf, 0, 0, 0, 0, string);
+	while (maxlen-- > 0) {
+		*buf++ = 0;
+	}
+	return buf;
 }
 
-TAGSTRING *tag_make_ref_len(TAGSTRING *buf, uint32 id128_1, uint32 id128_2, uint32 id128_3, uint32 id128_4, const char *string, int strlen)
+TAGSTRING *tag_make_sel(TAGSTRING *buf, int maxlen, const char *string)
 {
+	return tag_make_ref(buf, maxlen, 0, 0, 0, 0, string);
+}
+
+TAGSTRING *tag_make_ref_len(TAGSTRING *buf, int maxlen, uint32 id128_1, uint32 id128_2, uint32 id128_3, uint32 id128_4, const char *string, int strlen)
+{
+	if (maxlen <= (signed)sizeof (ht_tag_sel)) return tag_error(buf, maxlen);
+	if (maxlen <= (signed)sizeof (ht_tag_sel)+strlen) {
+		strlen = maxlen - sizeof (ht_tag_sel) - 1;
+	}
 	ht_tag_sel *tag=(ht_tag_sel*)buf;
 	tag->escape = '\e';
 	tag->magic = HT_TAG_SEL;
@@ -67,66 +79,70 @@ TAGSTRING *tag_make_ref_len(TAGSTRING *buf, uint32 id128_1, uint32 id128_2, uint
 	UNALIGNED_MOVE(tag->id128_3, id128_3);
 	UNALIGNED_MOVE(tag->id128_4, id128_4);
 	UNALIGNED_MOVE(tag->strlen, strlen);
-//	strcpy(buf+sizeof (ht_tag_sel), string);	/* FIXME: we assume that TAGSTRING is a char */
 	memcpy(buf+sizeof (ht_tag_sel), string, strlen);
-//	buf[sizeof (ht_tag_sel)+strlen]=0;
 	return buf+sizeof (ht_tag_sel)+strlen;
 }
 
-TAGSTRING *tag_make_ref(TAGSTRING *buf, uint32 id128_1, uint32 id128_2, uint32 id128_3, uint32 id128_4, const char *string)
+TAGSTRING *tag_make_ref(TAGSTRING *buf, int maxlen, uint32 id128_1, uint32 id128_2, uint32 id128_3, uint32 id128_4, const char *string)
 {
-	return tag_make_ref_len(buf, id128_1, id128_2, id128_3, id128_4, string, strlen(string));
+	return tag_make_ref_len(buf, maxlen, id128_1, id128_2, id128_3, id128_4, string, strlen(string));
 }
 
-TAGSTRING *tag_make_flags(TAGSTRING *buf, uint32 id, FileOfs ofs)
+TAGSTRING *tag_make_flags(TAGSTRING *buf, int maxlen, uint32 id, FileOfs ofs)
 {
-	ht_tag_flags *tag=(ht_tag_flags*)buf;
+	if (maxlen <= (signed)sizeof (ht_tag_flags)) return tag_error(buf, maxlen);
+	ht_tag_flags *tag = (ht_tag_flags*)buf;
 	tag->escape = '\e';
 	tag->magic = HT_TAG_FLAGS;
 	UNALIGNED_MOVE(tag->offset, ofs);
 	UNALIGNED_MOVE(tag->id, id);
-	return buf+sizeof (ht_tag_flags);
+	return buf + sizeof (ht_tag_flags);
 }
 
-TAGSTRING *tag_make_group(TAGSTRING *buf)
+TAGSTRING *tag_make_group(TAGSTRING *buf, int maxlen)
 {
-	ht_tag_group *tag=(ht_tag_group*)buf;
-	tag->escape='\e';
-	tag->magic=HT_TAG_GROUP;
-	return buf+sizeof (ht_tag_group);
+	if (maxlen <= (signed)sizeof (ht_tag_group)) return tag_error(buf, maxlen);
+	ht_tag_group *tag = (ht_tag_group*)buf;
+	tag->escape = '\e';
+	tag->magic = HT_TAG_GROUP;
+	return buf + sizeof (ht_tag_group);
 }
 
-TAGSTRING *tag_make_color(TAGSTRING *buf, uint32 color)
-{
-	ht_tag_color *tag=(ht_tag_color*)buf;
-	tag->escape='\e';
-	tag->magic=HT_TAG_COLOR;
-	UNALIGNED_MOVE(tag->color, color);
-	return buf+sizeof (ht_tag_color);
-}
-
-TAGSTRING *tag_make_default_color(TAGSTRING *buf)
+TAGSTRING *tag_make_color(TAGSTRING *buf, int maxlen, uint32 color)
 {
+	if (maxlen <= (signed)sizeof (ht_tag_color)) return tag_error(buf, maxlen);
 	ht_tag_color *tag = (ht_tag_color*)buf;
 	tag->escape = '\e';
 	tag->magic = HT_TAG_COLOR;
-	UNALIGNED_MOVE_CONST(tag->color, 0xffffffff, dword);
-	return buf+sizeof (ht_tag_color);
+	UNALIGNED_MOVE(tag->color, color);
+	return buf + sizeof (ht_tag_color);
 }
 
-TAGSTRING *tag_make_edit_byte(TAGSTRING *buf, FileOfs ofs)
+TAGSTRING *tag_make_default_color(TAGSTRING *buf, int maxlen)
 {
-	ht_tag_edit_byte *tag=(ht_tag_edit_byte*)buf;
-	tag->escape='\e';
-	tag->magic=HT_TAG_EDIT_BYTE;
+	if (maxlen <= (signed)sizeof (ht_tag_color)) return tag_error(buf, maxlen);
+	ht_tag_color *tag = (ht_tag_color*)buf;
+	tag->escape = '\e';
+	tag->magic = HT_TAG_COLOR;
+	UNALIGNED_MOVE_CONST(tag->color, 0xffffffff, uint32);
+	return buf + sizeof (ht_tag_color);
+}
+
+TAGSTRING *tag_make_edit_byte(TAGSTRING *buf, int maxlen, FileOfs ofs)
+{
+	if (maxlen <= (signed)sizeof (ht_tag_edit_byte)) return tag_error(buf, maxlen);
+	ht_tag_edit_byte *tag = (ht_tag_edit_byte*)buf;
+	tag->escape = '\e';
+	tag->magic = HT_TAG_EDIT_BYTE;
 	UNALIGNED_MOVE(tag->offset, ofs);
-	return buf+sizeof (ht_tag_edit_byte);
+	return buf + sizeof (ht_tag_edit_byte);
 }
 
-TAGSTRING *tag_make_edit_word(TAGSTRING *buf, FileOfs ofs, tag_endian e)
+TAGSTRING *tag_make_edit_word(TAGSTRING *buf, int maxlen, FileOfs ofs, tag_endian e)
 {
-	ht_tag_edit_word_generic *tag=(ht_tag_edit_word_generic*)buf;
-	tag->escape='\e';
+	if (maxlen <= (signed)sizeof (ht_tag_edit_word_generic)) return tag_error(buf, maxlen);
+	ht_tag_edit_word_generic *tag = (ht_tag_edit_word_generic*)buf;
+	tag->escape = '\e';
 	byte m = 0xff;
 	switch (e) {
 	case tag_endian_big:
@@ -139,16 +155,17 @@ TAGSTRING *tag_make_edit_word(TAGSTRING *buf, FileOfs ofs, tag_endian e)
 		m = HT_TAG_EDIT_WORD_VE;
 		break;
 	}
-	tag->magic=m;
+	tag->magic = m;
 	UNALIGNED_MOVE(tag->offset, ofs);
-	return buf+sizeof (ht_tag_edit_word_generic);
+	return buf + sizeof (ht_tag_edit_word_generic);
 }
 
-TAGSTRING *tag_make_edit_dword(TAGSTRING *buf, FileOfs ofs, tag_endian e)
+TAGSTRING *tag_make_edit_dword(TAGSTRING *buf, int maxlen, FileOfs ofs, tag_endian e)
 {
-	ht_tag_edit_dword_generic *tag=(ht_tag_edit_dword_generic*)buf;
-	tag->escape='\e';
-	byte m=0xff;
+	if (maxlen <= (signed)sizeof (ht_tag_edit_dword_generic)) return tag_error(buf, maxlen);
+	ht_tag_edit_dword_generic *tag = (ht_tag_edit_dword_generic*)buf;
+	tag->escape = '\e';
+	byte m = 0xff;
 	switch (e) {
 	case tag_endian_big:
 		m = HT_TAG_EDIT_DWORD_BE;
@@ -160,13 +177,14 @@ TAGSTRING *tag_make_edit_dword(TAGSTRING *buf, FileOfs ofs, tag_endian e)
 		m = HT_TAG_EDIT_DWORD_VE;
 		break;
 	}
-	tag->magic=m;
+	tag->magic = m;
 	UNALIGNED_MOVE(tag->offset, ofs);
-	return buf+sizeof (ht_tag_edit_dword_generic);
+	return buf + sizeof (ht_tag_edit_dword_generic);
 }
 
-TAGSTRING *tag_make_edit_qword(TAGSTRING *buf, FileOfs ofs, tag_endian e)
+TAGSTRING *tag_make_edit_qword(TAGSTRING *buf, int maxlen, FileOfs ofs, tag_endian e)
 {
+	if (maxlen <= (signed)sizeof (ht_tag_edit_qword_generic)) return tag_error(buf, maxlen);
 	ht_tag_edit_qword_generic *tag = (ht_tag_edit_qword_generic*)buf;
 	tag->escape = '\e';
 	byte m = 0xff;
@@ -186,8 +204,9 @@ TAGSTRING *tag_make_edit_qword(TAGSTRING *buf, FileOfs ofs, tag_endian e)
 	return buf + sizeof (ht_tag_edit_qword_generic);
 }
 
-TAGSTRING *tag_make_edit_time(TAGSTRING *buf, FileOfs ofs, tag_endian e)
+TAGSTRING *tag_make_edit_time(TAGSTRING *buf, int maxlen, FileOfs ofs, tag_endian e)
 {
+	if (maxlen <= (signed)sizeof (ht_tag_edit_time)) return tag_error(buf, maxlen);
 	ht_tag_edit_time *tag = (ht_tag_edit_time*)buf;
 	tag->escape = '\e';
 	byte m = 0xff;	
@@ -207,8 +226,9 @@ TAGSTRING *tag_make_edit_time(TAGSTRING *buf, FileOfs ofs, tag_endian e)
 	return buf + sizeof (ht_tag_edit_time);
 }
 
-TAGSTRING *tag_make_edit_char(TAGSTRING *buf, FileOfs ofs)
+TAGSTRING *tag_make_edit_char(TAGSTRING *buf, int maxlen, FileOfs ofs)
 {
+	if (maxlen <= (signed)sizeof (ht_tag_edit_char)) return tag_error(buf, maxlen);
 	ht_tag_edit_char *tag = (ht_tag_edit_char*)buf;
 	tag->escape = '\e';
 	tag->magic = HT_TAG_EDIT_CHAR;
@@ -216,41 +236,45 @@ TAGSTRING *tag_make_edit_char(TAGSTRING *buf, FileOfs ofs)
 	return buf + sizeof (ht_tag_edit_char);
 }
 
-TAGSTRING *tag_make_edit_bit(TAGSTRING *buf, FileOfs ofs, int bitidx)
+TAGSTRING *tag_make_edit_bit(TAGSTRING *buf, int maxlen, FileOfs ofs, int bitidx)
 {
-	ht_tag_edit_bit *tag=(ht_tag_edit_bit*)buf;
-	tag->escape='\e';
-	tag->magic=HT_TAG_EDIT_BIT;
+	if (maxlen <= (signed)sizeof (ht_tag_edit_bit)) return tag_error(buf, maxlen);
+	ht_tag_edit_bit *tag = (ht_tag_edit_bit*)buf;
+	tag->escape = '\e';
+	tag->magic = HT_TAG_EDIT_BIT;
 	UNALIGNED_MOVE(tag->offset, ofs);
 	UNALIGNED_MOVE(tag->bitidx, bitidx);
 	return buf+sizeof (ht_tag_edit_bit);
 }
 
-TAGSTRING *tag_make_edit_selvis(TAGSTRING *buf, FileOfs offset, char ch)
+TAGSTRING *tag_make_edit_selvis(TAGSTRING *buf, int maxlen, FileOfs offset, char ch)
 {
+	if (maxlen <= (signed)sizeof (ht_tag_edit_selvis)) return tag_error(buf, maxlen);
 	ht_tag_edit_selvis *tag=(ht_tag_edit_selvis*)buf;
-	tag->escape='\e';
-	tag->magic=HT_TAG_EDIT_SELVIS;
+	tag->escape = '\e';
+	tag->magic = HT_TAG_EDIT_SELVIS;
 	UNALIGNED_MOVE(tag->offset, offset);
-	tag->ch=ch;
-	return buf+sizeof (ht_tag_edit_selvis);
+	tag->ch = ch;
+	return buf + sizeof (ht_tag_edit_selvis);
 }
 
-TAGSTRING *tag_make_desc_byte(TAGSTRING *buf, FileOfs ofs32, uint32 id32)
+TAGSTRING *tag_make_desc_byte(TAGSTRING *buf, int maxlen, FileOfs ofs32, uint32 id32)
 {
-	ht_tag_desc_byte *tag=(ht_tag_desc_byte*)buf;
-	tag->escape='\e';
-	tag->magic=HT_TAG_DESC_BYTE;
+	if (maxlen <= (signed)sizeof (ht_tag_desc_byte)) return tag_error(buf, maxlen);
+	ht_tag_desc_byte *tag = (ht_tag_desc_byte*)buf;
+	tag->escape = '\e';
+	tag->magic = HT_TAG_DESC_BYTE;
 	UNALIGNED_MOVE(tag->offset, ofs32);
 	UNALIGNED_MOVE(tag->id, id32);
-	return buf+sizeof (ht_tag_desc_byte);
+	return buf + sizeof (ht_tag_desc_byte);
 }
 
-TAGSTRING *tag_make_desc_word(TAGSTRING *buf, FileOfs ofs32, uint32 id32, tag_endian e)
+TAGSTRING *tag_make_desc_word(TAGSTRING *buf, int maxlen, FileOfs ofs32, uint32 id32, tag_endian e)
 {
-	ht_tag_desc_word_generic *tag=(ht_tag_desc_word_generic*)buf;
-	tag->escape='\e';
-	byte m=0xff;
+	if (maxlen <= (signed)sizeof (ht_tag_desc_word_generic)) return tag_error(buf, maxlen);
+	ht_tag_desc_word_generic *tag = (ht_tag_desc_word_generic*)buf;
+	tag->escape = '\e';
+	byte m = 0xff;
 	switch (e) {
 	case tag_endian_big:
 		m = HT_TAG_DESC_WORD_BE;
@@ -262,17 +286,18 @@ TAGSTRING *tag_make_desc_word(TAGSTRING *buf, FileOfs ofs32, uint32 id32, tag_en
 		m = HT_TAG_DESC_WORD_VE;
 		break;
 	}
-	tag->magic=m;
+	tag->magic = m;
 	UNALIGNED_MOVE(tag->offset, ofs32);
 	UNALIGNED_MOVE(tag->id, id32);
-	return buf+sizeof (ht_tag_desc_word_generic);
+	return buf + sizeof (ht_tag_desc_word_generic);
 }
 
-TAGSTRING *tag_make_desc_dword(TAGSTRING *buf, FileOfs ofs32, uint32 id32, tag_endian e)
+TAGSTRING *tag_make_desc_dword(TAGSTRING *buf, int maxlen, FileOfs ofs32, uint32 id32, tag_endian e)
 {
-	ht_tag_desc_dword_generic *tag=(ht_tag_desc_dword_generic*)buf;
-	tag->escape='\e';
-	byte m=0xff;
+	if (maxlen <= (signed)sizeof (ht_tag_desc_dword_generic)) return tag_error(buf, maxlen);
+	ht_tag_desc_dword_generic *tag = (ht_tag_desc_dword_generic*)buf;
+	tag->escape = '\e';
+	byte m = 0xff;
 	switch (e) {
 	case tag_endian_big:
 		m = HT_TAG_DESC_DWORD_BE;
@@ -284,17 +309,18 @@ TAGSTRING *tag_make_desc_dword(TAGSTRING *buf, FileOfs ofs32, uint32 id32, tag_e
 		m = HT_TAG_DESC_DWORD_VE;
 		break;
 	}
-	tag->magic=m;
+	tag->magic = m;
 	UNALIGNED_MOVE(tag->offset, ofs32);
 	UNALIGNED_MOVE(tag->id, id32);
-	return buf+sizeof (ht_tag_desc_dword_generic);
+	return buf + sizeof (ht_tag_desc_dword_generic);
 }
 
-TAGSTRING *tag_make_desc_qword(TAGSTRING *buf, FileOfs ofs32, uint32 id32, tag_endian e)
+TAGSTRING *tag_make_desc_qword(TAGSTRING *buf, int maxlen, FileOfs ofs32, uint32 id32, tag_endian e)
 {
-	ht_tag_desc_qword_generic *tag=(ht_tag_desc_qword_generic*)buf;
-	tag->escape='\e';
-	byte m=0xff;
+	if (maxlen <= (signed)sizeof (ht_tag_desc_qword_generic)) return tag_error(buf, maxlen);
+	ht_tag_desc_qword_generic *tag = (ht_tag_desc_qword_generic*)buf;
+	tag->escape = '\e';
+	byte m = 0xff;
 	switch (e) {
 	case tag_endian_big:
 		m = HT_TAG_DESC_QWORD_BE;
@@ -306,200 +332,209 @@ TAGSTRING *tag_make_desc_qword(TAGSTRING *buf, FileOfs ofs32, uint32 id32, tag_e
 		m = HT_TAG_DESC_QWORD_VE;
 		break;
 	}
-	tag->magic=m;
+	tag->magic = m;
 	UNALIGNED_MOVE(tag->offset, ofs32);
 	UNALIGNED_MOVE(tag->id, id32);
-	return buf+sizeof (ht_tag_desc_qword_generic);
+	return buf + sizeof (ht_tag_desc_qword_generic);
 }
 
 /**/
 
-void statictag_to_tag(const char *statictag_str, TAGSTRING *tag_str, uint64 relocation, bool std_bigendian)
+void statictag_to_tag(const char *statictag_str, TAGSTRING *tag_str, int maxlen, uint64 relocation, bool std_bigendian)
 {
+	if (maxlen < 1) return;
+	if (maxlen == 1) {
+		*tag_str = 0;
+		return;
+	}
 	FileOfs ofs = 0;
 	ID id;
+	TAGSTRING *tag_str_end = tag_str + maxlen - 1; 
 	while (*statictag_str) {
-		if (*statictag_str=='\e') {
+		if (*statictag_str == '\e') {
 			switch ((byte)*(statictag_str+1)) {
 			case HT_STATICTAG_EDIT_BYTE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_byte(tag_str, ofs+relocation);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_byte(tag_str, tag_str_end-tag_str, ofs+relocation);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_WORD_LE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_word(tag_str, ofs+relocation, tag_endian_little);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_word(tag_str, tag_str_end-tag_str, ofs+relocation, tag_endian_little);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_DWORD_LE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_dword(tag_str, ofs+relocation, tag_endian_little);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_dword(tag_str, tag_str_end-tag_str, ofs+relocation, tag_endian_little);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_QWORD_LE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_qword(tag_str, ofs+relocation, tag_endian_little);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_qword(tag_str, tag_str_end-tag_str, ofs+relocation, tag_endian_little);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_WORD_BE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_word(tag_str, ofs+relocation, tag_endian_big);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_word(tag_str, tag_str_end-tag_str, ofs+relocation, tag_endian_big);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_DWORD_BE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_dword(tag_str, ofs+relocation, tag_endian_big);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_dword(tag_str, tag_str_end-tag_str, ofs+relocation, tag_endian_big);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_QWORD_BE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_qword(tag_str, ofs+relocation, tag_endian_big);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_qword(tag_str, tag_str_end-tag_str, ofs+relocation, tag_endian_big);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_WORD_VE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_word(tag_str, ofs+relocation, std_bigendian ? tag_endian_big : tag_endian_little);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_word(tag_str, tag_str_end-tag_str, ofs+relocation, std_bigendian ? tag_endian_big : tag_endian_little);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_DWORD_VE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_dword(tag_str, ofs+relocation, std_bigendian ? tag_endian_big : tag_endian_little);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_dword(tag_str, tag_str_end-tag_str, ofs+relocation, std_bigendian ? tag_endian_big : tag_endian_little);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_QWORD_VE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_qword(tag_str, ofs+relocation, std_bigendian ? tag_endian_big : tag_endian_little);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_qword(tag_str, tag_str_end-tag_str, ofs+relocation, std_bigendian ? tag_endian_big : tag_endian_little);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_TIME_LE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_time(tag_str, ofs+relocation, tag_endian_little);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_time(tag_str, tag_str_end-tag_str, ofs+relocation, tag_endian_little);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_TIME_BE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_time(tag_str, ofs+relocation, tag_endian_big);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_time(tag_str, tag_str_end-tag_str, ofs+relocation, tag_endian_big);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_TIME_VE:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_time(tag_str, ofs+relocation, std_bigendian ? tag_endian_big : tag_endian_little);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_time(tag_str, tag_str_end-tag_str, ofs+relocation, std_bigendian ? tag_endian_big : tag_endian_little);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_CHAR:
-				ofs=hexd(statictag_str+2);
-				tag_str=tag_make_edit_char(tag_str, ofs+relocation);
-				statictag_str+=2+8;
+				ofs = hexd(statictag_str+2);
+				tag_str = tag_make_edit_char(tag_str, tag_str_end-tag_str, ofs+relocation);
+				statictag_str += 2+8;
 				break;
 			case HT_STATICTAG_EDIT_BIT: {
-				ofs=hexd(statictag_str+2);
-				int bitidx=hexb(statictag_str+2+8);
-				tag_str=tag_make_edit_bit(tag_str, ofs+relocation, bitidx);
-				statictag_str+=2+8+2;
+				ofs = hexd(statictag_str+2);
+				int bitidx = hexb(statictag_str+2+8);
+				tag_str = tag_make_edit_bit(tag_str, tag_str_end-tag_str, ofs+relocation, bitidx);
+				statictag_str += 2+8+2;
 				break;
 			}
 			case HT_STATICTAG_EDIT_SELVIS: {
-				ofs=hexd(statictag_str+2);
-				char ch=hexb(statictag_str+2+8);
-				tag_str=tag_make_edit_selvis(tag_str, ofs+relocation, ch);
-				statictag_str+=2+8+2;
+				ofs = hexd(statictag_str+2);
+				char ch = hexb(statictag_str+2+8);
+				tag_str = tag_make_edit_selvis(tag_str, tag_str_end-tag_str, ofs+relocation, ch);
+				statictag_str += 2+8+2;
 				break;
 			}
 			case HT_STATICTAG_DESC_BYTE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_byte(tag_str, ofs+relocation, id);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_byte(tag_str, tag_str_end-tag_str, ofs+relocation, id);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_DESC_WORD_LE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_word(tag_str, ofs+relocation, id, tag_endian_little);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_word(tag_str, tag_str_end-tag_str, ofs+relocation, id, tag_endian_little);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_DESC_DWORD_LE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_dword(tag_str, ofs+relocation, id, tag_endian_little);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_dword(tag_str, tag_str_end-tag_str, ofs+relocation, id, tag_endian_little);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_DESC_QWORD_LE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_qword(tag_str, ofs+relocation, id, tag_endian_little);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_qword(tag_str, tag_str_end-tag_str, ofs+relocation, id, tag_endian_little);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_DESC_WORD_BE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_word(tag_str, ofs+relocation, id, tag_endian_big);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_word(tag_str, tag_str_end-tag_str, ofs+relocation, id, tag_endian_big);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_DESC_DWORD_BE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_dword(tag_str, ofs+relocation, id, tag_endian_big);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_dword(tag_str, tag_str_end-tag_str, ofs+relocation, id, tag_endian_big);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_DESC_QWORD_BE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_qword(tag_str, ofs+relocation, id, tag_endian_big);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_qword(tag_str, tag_str_end-tag_str, ofs+relocation, id, tag_endian_big);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_DESC_WORD_VE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_word(tag_str, ofs+relocation, id, std_bigendian ? tag_endian_big : tag_endian_little);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_word(tag_str, tag_str_end-tag_str, ofs+relocation, id, std_bigendian ? tag_endian_big : tag_endian_little);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_DESC_DWORD_VE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_dword(tag_str, ofs+relocation, id, std_bigendian ? tag_endian_big : tag_endian_little);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_dword(tag_str, tag_str_end-tag_str, ofs+relocation, id, std_bigendian ? tag_endian_big : tag_endian_little);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_DESC_QWORD_VE:
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_desc_qword(tag_str, ofs+relocation, id, std_bigendian ? tag_endian_big : tag_endian_little);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_desc_qword(tag_str, tag_str_end-tag_str, ofs+relocation, id, std_bigendian ? tag_endian_big : tag_endian_little);
+				statictag_str += 2+8+8;
 				break;
 			case HT_STATICTAG_SEL: {
-				uint32 id_1=hexd(statictag_str+2);
-				uint32 id_2=hexd(statictag_str+2+8);
-				uint32 id_3=hexd(statictag_str+2+16);
-				uint32 id_4=hexd(statictag_str+2+24);
-				byte len=hexb(statictag_str+2+8+8+8+8);
-				tag_str=tag_make_ref_len(tag_str, id_1, id_2, id_3, id_4, statictag_str+2+8+8+8+8+2, len);
-				statictag_str+=2+8+8+8+8+2+len;
+				uint32 id_1 = hexd(statictag_str+2);
+				uint32 id_2 = hexd(statictag_str+2+8);
+				uint32 id_3 = hexd(statictag_str+2+16);
+				uint32 id_4 = hexd(statictag_str+2+24);
+				byte len = hexb(statictag_str+2+8+8+8+8);
+				tag_str = tag_make_ref_len(tag_str, tag_str_end-tag_str, id_1, id_2, id_3, id_4, statictag_str+2+8+8+8+8+2, len);
+				statictag_str += 2+8+8+8+8+2+len;
 				break;
 			}
 			case HT_STATICTAG_FLAGS: {
-				ofs=hexd(statictag_str+2);
-				id=hexd(statictag_str+2+8);
-				tag_str=tag_make_flags(tag_str, id, ofs+relocation);
-				statictag_str+=2+8+8;
+				ofs = hexd(statictag_str+2);
+				id = hexd(statictag_str+2+8);
+				tag_str = tag_make_flags(tag_str, tag_str_end-tag_str, id, ofs+relocation);
+				statictag_str += 2+8+8;
 				break;
 			}
 			case HT_STATICTAG_GROUP:
-				tag_str=tag_make_group(tag_str);
-				statictag_str+=2;
+				tag_str = tag_make_group(tag_str, tag_str_end-tag_str);
+				statictag_str += 2;
 				break;
 			case HT_STATICTAG_COLOR: {
-				byte color=hexb(statictag_str+2);
-				tag_str=tag_make_color(tag_str, color);
-				statictag_str+=2+2;
+				byte color = hexb(statictag_str+2);
+				tag_str = tag_make_color(tag_str, tag_str_end-tag_str, color);
+				statictag_str += 2+2;
 				break;
 			}
 			default:
 				HT_ERROR("error in statictag string!");
 			}
 		} else {
-			*(tag_str++) = *(statictag_str++);
+			if (tag_str < tag_str_end) {
+				*tag_str++ = *statictag_str;
+			}
+			statictag_str++;
 		}
 	}
-	*tag_str=0;
+	*tag_str = 0;
 }
 
 TAGSTRING *tag_findnext(const TAGSTRING *tagstring)
@@ -619,7 +654,7 @@ int tag_get_vlen(const TAGSTRING *tagstring)
 	}
 }
 
-int time_mp[14]={0,1,3,4,6,7,9,10,12,13,15,16,17,18};
+static int time_mp[14] = {0,1,3,4,6,7,9,10,12,13,15,16,17,18};
 
 int tag_get_micropos(const TAGSTRING *tagstring, int i)
 {
@@ -806,7 +841,7 @@ void tag_get_id(const TAGSTRING *tagstring, uint32 *id128_1, uint32 *id128_2, ui
 
 TAGSTRING *tag_get_seltext(const TAGSTRING *tagstring)
 {
-	return (TAGSTRING *)(tagstring+sizeof(ht_tag_sel));
+	return (TAGSTRING *)(tagstring + sizeof(ht_tag_sel));
 }
 
 int tag_get_seltextlen(const TAGSTRING *tagstring)
@@ -911,25 +946,43 @@ bool tag_is_editable(const TAGSTRING *tagstring)
 	return false;
 }
 
-void tag_strcat(TAGSTRING *dest, const TAGSTRING *src)
+void tag_strcat(TAGSTRING *dest, int maxlen, const TAGSTRING *src)
 {
 	int l = tag_strlen(dest);
-	tag_strcpy(dest+l, src);
+	tag_strcpy(dest+l, maxlen-l, src);
 }
 
-void tag_strcpy(TAGSTRING *dest, const TAGSTRING *src)
+void tag_strcpy(TAGSTRING *dest, int maxlen, const TAGSTRING *src)
 {
-	int l = tag_strlen(src);
-	memcpy(dest, src, l);
-	dest[l] = 0;
+	if (maxlen > 0) {
+		maxlen--;
+		while (*src && maxlen) {
+			if (*src == '\e') {
+				int l = tag_get_len(src);
+				if (l > maxlen) {
+					tag_error(dest, maxlen);
+					return;
+				} else {
+					memcpy(dest, src, l);
+					dest += l;
+					src += l;
+					maxlen -= l;
+				}
+			} else {
+				*dest++ = *src++;
+				maxlen--;
+			}
+		}
+		*dest = 0;
+	}
 }
 
 TAGSTRING *tag_strdup(const TAGSTRING *tagstring)
 {
 	if (!tagstring) return NULL;
 	int l = tag_strlen(tagstring);
-	TAGSTRING *s = ht_malloc((sizeof (TAGSTRING))*l + 1);
-	memcpy(s, tagstring, sizeof (TAGSTRING) * l);
+	TAGSTRING *s = ht_malloc(l + 1);
+	memcpy(s, tagstring, l);
 	s[l] = 0;
 	return s;
 }
@@ -953,137 +1006,137 @@ int tag_strvlen(const TAGSTRING *tagstring)
 {
 	int c=0, r, v;
 	while (*tagstring) {
-		if (tagstring[0]=='\e') {
-			r=tag_get_len(tagstring);
-			v=tag_get_vlen(tagstring);
+		if (tagstring[0] == '\e') {
+			r = tag_get_len(tagstring);
+			v = tag_get_vlen(tagstring);
 		} else {
-			r=1;
-			v=1;
+			r = 1;
+			v = 1;
 		}
-		tagstring+=r;
-		c+=v;
+		tagstring += r;
+		c += v;
 	}
 	return c;
 }
 
 int tag_count_selectable_tags_in_group(const TAGSTRING *tagstring, int group)
 {
-	int c=0;
+	int c = 0;
 	tagstring = tag_get_group(tagstring, group);
 	while ((tagstring = tag_findnext(tagstring))) {
 		switch (tagstring[1]) {
 		case HT_TAG_EDIT_BYTE:
 			c++;
-			tagstring+=HT_TAG_EDIT_BYTE_LEN;
+			tagstring += HT_TAG_EDIT_BYTE_LEN;
 			break;
 		case HT_TAG_EDIT_WORD_LE:
 			c++;
-			tagstring+=HT_TAG_EDIT_WORD_LE_LEN;
+			tagstring += HT_TAG_EDIT_WORD_LE_LEN;
 			break;
 		case HT_TAG_EDIT_DWORD_LE:
 			c++;
-			tagstring+=HT_TAG_EDIT_DWORD_LE_LEN;
+			tagstring += HT_TAG_EDIT_DWORD_LE_LEN;
 			break;
 		case HT_TAG_EDIT_QWORD_LE:
 			c++;
-			tagstring+=HT_TAG_EDIT_QWORD_LE_LEN;
+			tagstring += HT_TAG_EDIT_QWORD_LE_LEN;
 			break;
 		case HT_TAG_EDIT_WORD_BE:
 			c++;
-			tagstring+=HT_TAG_EDIT_WORD_BE_LEN;
+			tagstring += HT_TAG_EDIT_WORD_BE_LEN;
 			break;
 		case HT_TAG_EDIT_DWORD_BE:
 			c++;
-			tagstring+=HT_TAG_EDIT_DWORD_BE_LEN;
+			tagstring += HT_TAG_EDIT_DWORD_BE_LEN;
 			break;
 		case HT_TAG_EDIT_QWORD_BE:
 			c++;
-			tagstring+=HT_TAG_EDIT_QWORD_BE_LEN;
+			tagstring += HT_TAG_EDIT_QWORD_BE_LEN;
 			break;
 		case HT_TAG_EDIT_WORD_VE:
 			c++;
-			tagstring+=HT_TAG_EDIT_WORD_VE_LEN;
+			tagstring += HT_TAG_EDIT_WORD_VE_LEN;
 			break;
 		case HT_TAG_EDIT_DWORD_VE:
 			c++;
-			tagstring+=HT_TAG_EDIT_DWORD_VE_LEN;
+			tagstring += HT_TAG_EDIT_DWORD_VE_LEN;
 			break;
 		case HT_TAG_EDIT_QWORD_VE:
 			c++;
-			tagstring+=HT_TAG_EDIT_QWORD_VE_LEN;
+			tagstring += HT_TAG_EDIT_QWORD_VE_LEN;
 			break;
 		case HT_TAG_EDIT_TIME_LE:
 			c++;
-			tagstring+=HT_TAG_EDIT_TIME_LE_LEN;
+			tagstring += HT_TAG_EDIT_TIME_LE_LEN;
 			break;
 		case HT_TAG_EDIT_TIME_BE:
 			c++;
-			tagstring+=HT_TAG_EDIT_TIME_BE_LEN;
+			tagstring += HT_TAG_EDIT_TIME_BE_LEN;
 			break;
 		case HT_TAG_EDIT_TIME_VE:
 			c++;
-			tagstring+=HT_TAG_EDIT_TIME_VE_LEN;
+			tagstring += HT_TAG_EDIT_TIME_VE_LEN;
 			break;
 		case HT_TAG_EDIT_CHAR:
 			c++;
-			tagstring+=HT_TAG_EDIT_CHAR_LEN;
+			tagstring += HT_TAG_EDIT_CHAR_LEN;
 			break;
 		case HT_TAG_EDIT_BIT:
 			c++;
-			tagstring+=HT_TAG_EDIT_BIT_LEN;
+			tagstring += HT_TAG_EDIT_BIT_LEN;
 			break;
 		case HT_TAG_SEL:
 			c++;
-			tagstring+=HT_TAG_SEL_LEN(((ht_tag_sel*)tagstring)->strlen);
+			tagstring += HT_TAG_SEL_LEN(((ht_tag_sel*)tagstring)->strlen);
 			break;
 		case HT_TAG_FLAGS:
 			c++;
-			tagstring+=HT_TAG_FLAGS_LEN;
+			tagstring += HT_TAG_FLAGS_LEN;
 			break;
 		case HT_TAG_GROUP:
 			return c;
 		case HT_TAG_DESC_BYTE:
 			c++;
-			tagstring+=HT_TAG_DESC_BYTE_LEN;
+			tagstring += HT_TAG_DESC_BYTE_LEN;
 			break;
 		case HT_TAG_DESC_WORD_LE:
 			c++;
-			tagstring+=HT_TAG_DESC_WORD_LE_LEN;
+			tagstring += HT_TAG_DESC_WORD_LE_LEN;
 			break;
 		case HT_TAG_DESC_DWORD_LE:
 			c++;
-			tagstring+=HT_TAG_DESC_DWORD_LE_LEN;
+			tagstring += HT_TAG_DESC_DWORD_LE_LEN;
 			break;
 		case HT_TAG_DESC_QWORD_LE:
 			c++;
-			tagstring+=HT_TAG_DESC_QWORD_LE_LEN;
+			tagstring += HT_TAG_DESC_QWORD_LE_LEN;
 			break;
 		case HT_TAG_DESC_WORD_BE:
 			c++;
-			tagstring+=HT_TAG_DESC_WORD_BE_LEN;
+			tagstring += HT_TAG_DESC_WORD_BE_LEN;
 			break;
 		case HT_TAG_DESC_DWORD_BE:
 			c++;
-			tagstring+=HT_TAG_DESC_DWORD_BE_LEN;
+			tagstring += HT_TAG_DESC_DWORD_BE_LEN;
 			break;
 		case HT_TAG_DESC_QWORD_BE:
 			c++;
-			tagstring+=HT_TAG_DESC_QWORD_BE_LEN;
+			tagstring += HT_TAG_DESC_QWORD_BE_LEN;
 			break;
 		case HT_TAG_DESC_WORD_VE:
 			c++;
-			tagstring+=HT_TAG_DESC_WORD_VE_LEN;
+			tagstring += HT_TAG_DESC_WORD_VE_LEN;
 			break;
 		case HT_TAG_DESC_DWORD_VE:
 			c++;
-			tagstring+=HT_TAG_DESC_DWORD_VE_LEN;
+			tagstring += HT_TAG_DESC_DWORD_VE_LEN;
 			break;
 		case HT_TAG_DESC_QWORD_VE:
 			c++;
-			tagstring+=HT_TAG_DESC_QWORD_VE_LEN;
+			tagstring += HT_TAG_DESC_QWORD_VE_LEN;
 			break;
 		default:
-			tagstring+=tag_get_len(tagstring);
+			tagstring += tag_get_len(tagstring);
 			break;
 		}
 	}
@@ -1213,13 +1266,13 @@ int tag_count_selectable_tags(const TAGSTRING *tagstring)
 
 int tag_count_groups(const TAGSTRING *tagstring)
 {
-	int c=1;
-	while ((tagstring=tag_findnext(tagstring))) {
-		if (tagstring[1]==HT_TAG_GROUP) {
+	int c = 1;
+	while ((tagstring = tag_findnext(tagstring))) {
+		if (tagstring[1] == HT_TAG_GROUP) {
 			c++;
-			tagstring+=HT_TAG_GROUP_LEN;
+			tagstring += HT_TAG_GROUP_LEN;
 		} else {
-			tagstring+=tag_get_len(tagstring);
+			tagstring += tag_get_len(tagstring);
 		}
 	}
 	return c;
@@ -1227,10 +1280,10 @@ int tag_count_groups(const TAGSTRING *tagstring)
 
 TAGSTRING *tag_get_selectable_tag(const TAGSTRING *tagstring, int n, int group)
 {
-	const TAGSTRING *r=NULL;
-	if (group>0) tagstring=tag_get_group(tagstring, group);
+	const TAGSTRING *r = NULL;
+	if (group > 0) tagstring = tag_get_group(tagstring, group);
 	n++;
-	while ((n) && (tagstring=tag_findnext(tagstring))) {
+	while (n && (tagstring = tag_findnext(tagstring))) {
 		switch (tagstring[1]) {
 			case HT_TAG_EDIT_BYTE:
 				n--;
@@ -1400,37 +1453,37 @@ TAGSTRING *tag_get_group(const TAGSTRING *tagstring, int group)
 int tag_get_class(const TAGSTRING *tagstring)
 {
 	switch (tagstring[1]) {
-		case HT_TAG_EDIT_BYTE:
-		case HT_TAG_EDIT_WORD_LE:
-		case HT_TAG_EDIT_DWORD_LE:
-		case HT_TAG_EDIT_QWORD_LE:
-		case HT_TAG_EDIT_WORD_BE:
-		case HT_TAG_EDIT_DWORD_BE:
-		case HT_TAG_EDIT_QWORD_BE:
-		case HT_TAG_EDIT_WORD_VE:
-		case HT_TAG_EDIT_DWORD_VE:
-		case HT_TAG_EDIT_QWORD_VE:
-		case HT_TAG_EDIT_TIME_LE:
-		case HT_TAG_EDIT_TIME_BE:
-		case HT_TAG_EDIT_TIME_VE:
-		case HT_TAG_EDIT_CHAR:
-		case HT_TAG_EDIT_BIT:
-			return tag_class_edit;
-		case HT_TAG_SEL:
-		case HT_TAG_FLAGS:
-		case HT_TAG_DESC_BYTE:
-		case HT_TAG_DESC_WORD_LE:
-		case HT_TAG_DESC_DWORD_LE:
-		case HT_TAG_DESC_QWORD_LE:
-		case HT_TAG_DESC_WORD_BE:
-		case HT_TAG_DESC_DWORD_BE:
-		case HT_TAG_DESC_QWORD_BE:
-		case HT_TAG_DESC_WORD_VE:
-		case HT_TAG_DESC_DWORD_VE:
-		case HT_TAG_DESC_QWORD_VE:
-			return tag_class_sel;
-		default:
-			return tag_class_no;
+	case HT_TAG_EDIT_BYTE:
+	case HT_TAG_EDIT_WORD_LE:
+	case HT_TAG_EDIT_DWORD_LE:
+	case HT_TAG_EDIT_QWORD_LE:
+	case HT_TAG_EDIT_WORD_BE:
+	case HT_TAG_EDIT_DWORD_BE:
+	case HT_TAG_EDIT_QWORD_BE:
+	case HT_TAG_EDIT_WORD_VE:
+	case HT_TAG_EDIT_DWORD_VE:
+	case HT_TAG_EDIT_QWORD_VE:
+	case HT_TAG_EDIT_TIME_LE:
+	case HT_TAG_EDIT_TIME_BE:
+	case HT_TAG_EDIT_TIME_VE:
+	case HT_TAG_EDIT_CHAR:
+	case HT_TAG_EDIT_BIT:
+		return tag_class_edit;
+	case HT_TAG_SEL:
+	case HT_TAG_FLAGS:
+	case HT_TAG_DESC_BYTE:
+	case HT_TAG_DESC_WORD_LE:
+	case HT_TAG_DESC_DWORD_LE:
+	case HT_TAG_DESC_QWORD_LE:
+	case HT_TAG_DESC_WORD_BE:
+	case HT_TAG_DESC_DWORD_BE:
+	case HT_TAG_DESC_QWORD_BE:
+	case HT_TAG_DESC_WORD_VE:
+	case HT_TAG_DESC_DWORD_VE:
+	case HT_TAG_DESC_QWORD_VE:
+		return tag_class_sel;
+	default:
+		return tag_class_no;
 	}
 }
 
@@ -1445,12 +1498,12 @@ char *tag_striptags(char *dest, const TAGSTRING *src)
 	while (*src) {
 		if (src[0]=='\e') {
 			switch (src[1]) {
-				case HT_TAG_SEL: {
-					src += sizeof (ht_tag_sel);
-					break;
-				}
-				default:
-					src+=tag_get_len(src);
+			case HT_TAG_SEL: {
+				src += sizeof (ht_tag_sel);
+				break;
+			}
+			default:
+				src += tag_get_len(src);
 			}
 		} else {
 			*dest = *src;
